(in-package "USER")

(defun start ()
    ;(load "board.lsp")
    ;(load "score.lsp")
    ;(load "movable.lsp")
    (setf *quit* nil)
    (format t "Welcome to Jangki. (Korean Chess)~%")
    (let ((input nil))
        (loop do (setf input (get-move))
	      until (end-of-game input)
	)
    )
    "Bye!!!"
)

(defun get-move ()
    (format t "~%Please give me a command for moving a piece.~%")
    (let ((lst (read))(current nil))
	;(format t "Input data : ~S~%" lst)
	(cond
	    ( (equal (first lst) 'show) (show) )
	    ( (equal (first lst) 'quit) (setf *quit* *end*) nil)
	    ( t (setf current (from-piece (first lst)))
		(if (null current)
		    (unknown-message lst)
	            (if (valid-piece current (second lst) :p (first lst))
	                (list current (to-piece (first (last lst))))
	                nil
		    )
		)
	    )
	)
    )
)

(defun unknown-message (lst)
    (format t "I cannot understand ~S.~%" lst)
)

(defun valid-piece (position ele &key p)
    (cond
	;( (null position) (no-position-message position) nil )
        ( (not (symbolp ele)) (no-piece-message ele) nil )
	( (is-same (get-piece position) ele) t )
	( t (no-piece-message ele p) nil )
    )
)

;(defun no-position-message (p)
;    (format t "The position ~S~% cannot be understood." p)
;) 

(defun no-piece-message (ele &optional position)
    (format t "There is no such piece, ~S" ele)
    (if position
	(format t " at ~S.~%" position)
	(format t ".~%")
    )
)

(defun get-piece (lst)
    (aref *board* (first lst) (second lst))
)

(defun is-same (num ch)
    (if (evenp num)
	(eq (1- num) (get-id ch))
	(eq num (get-id ch))
    )
)

(defun get-id (ch)
    (cond
	( (eq ch 'R) 1 )
	( (eq ch 'N) 3 )
	( (eq ch 'B) 5 )
	( (eq ch 'G) 7 )
	( (eq ch 'K) 9 )
	( (eq ch 'C) 11)
	( (eq ch 'P) 13)
    )
)

(defun from-piece (word)
    (let ((x nil)(y nil))
        (setf x (get-x (char (string word) 0)))
	(setf y (get-y (string word) (length (string word))))
	(if (or (null x) (null y))
	    nil
	    (list x y)
	)
    )
)

(defun get-x (ch)
    (cond
        ( (eq ch #\A) 0 )
        ( (eq ch #\B) 1 )
        ( (eq ch #\C) 2 )
        ( (eq ch #\D) 3 )
        ( (eq ch #\E) 4 )
        ( (eq ch #\F) 5 )
        ( (eq ch #\G) 6 )
        ( (eq ch #\H) 7 )
        ( (eq ch #\I) 8 )
        ( t nil )
    )
)

(defun get-y (str num)
    (cond
	( (eq num 2) (get-num (char str 1)) )
	( t nil )
    )
)

(defun get-num (ch)
    (cond
	( (eq ch #\0) 0 )
	( (eq ch #\1) 1 )
	( (eq ch #\2) 2 )
	( (eq ch #\3) 3 )
	( (eq ch #\4) 4 )
	( (eq ch #\5) 5 )
	( (eq ch #\6) 6 )
	( (eq ch #\7) 7 )
	( (eq ch #\8) 8 )
	( (eq ch #\9) 9 )
	( t nil )
    )
)

(defun to-piece (ele)
    (from-piece ele)
)

(defun end-of-game (lst)
    (if lst
        (let ((object nil))
	    (if (setf object (piece-in lst))
	        (computer-move object)
	    )
	)
    )
    (if (terminate) t nil)
)

; lst : '((x y) (x y))
(defun piece-in (lst)
    (let ((result nil))
        (if (member (second lst)
		    (movable (a-piece (first lst)))
		    :test #'equal
	    )
	    (progn
		(human-move lst)
		(setf result (a-piece (second lst)))
	    )
	    (unmovable-message)
	)
	result
    )
)

; ((x y) piece)
(defun a-piece (lst)
    (list lst (get-piece lst) )
)

(defun unmovable-message ()
    (format t "You cannot move it to that position.~%")
)

; lst : '((x y) (x y))
(defun human-move (lst &key r pseudo)
    (let ((piece nil))
	(cond
	     ( r (setf piece (get-piece (second lst))) )
	     ( pseudo
		 (push (get-piece (second lst)) *stack*)
		 (setf piece (get-piece (first lst)))
	     )
	     ( t (setf piece (get-piece (first lst))) )
	)

	;(format t "h*M:lst~S ~S ~S~%" lst
	;    (aref *board* (first (first lst)) (second (first lst)))
	;    (aref *board* (first (second lst)) (second (second lst)))
	;)

	(setf (aref *board*
		    (first (first lst))
		    (second (first lst))
	      )
	      (if r piece 0)
	)
	(setf (aref *board*
		    (first (second lst))
		    (second (second lst))
	      )
	      (if r (pop *stack*) piece)
	) 
	(if (not pseudo) (setf *history* (append *history* lst)))
    )
    lst
)

; lst : '((x y) p)
(defun computer-move (lst)
    ;(if (null lst) (setf *quit* t) )
    ;(format t "You moved ~S~%" lst)
    (format t "Thinking...")
    (let ((result nil))
        (if (< *turn* 6)
	    (setf result (deploy-heuristic lst))
	    (progn
	        (multiple-value-setq (rank result)
		    (minimax
		        (grow-tree '((x)) *look-ahead* *red*)
		        *look-ahead*
		    )
		)
		(setf result
		    (append (human-move (second result))
			    (list (get-piece (first (first result))))
		    )
		)
	    )
	)
	(format t "Done.~%")
	(show-move result)
    )
    (setf *turn* (1+ *turn*))
)

; lst '((x y) (x y) p [p])
(defun show-move (lst)
    (if lst
    (let (start end name)
	;(format t "show-move : ~S~%" lst)
	(setf start (num-to-str (first lst)))
	(setf end (num-to-str (second lst)))
	(setf name (get-name (third lst)))
	(format t "I moved ~A from ~A to ~A" name start end)
	(if (fourth lst)
	    (format t " and catched ~A" (get-name (fourth lst)))
	)
	(format t ".~%")
    )
    )
)

(defun num-to-str (lst)
    (concatenate 'string
		 (string (num-to-char (first lst)))
		 (string (num-to-name (second lst)))
    )
)

(defun num-to-char (num)
    (cond
	( (= 0 num) 'A )
	( (= 1 num) 'B )
	( (= 2 num) 'C )
	( (= 3 num) 'D )
	( (= 4 num) 'E )
	( (= 5 num) 'F )
	( (= 6 num) 'G )
	( (= 7 num) 'H )
	( (= 8 num) 'I )
	( t (format t "unknown cordination ~D~%" num) 'X)
    )
)

(defun num-to-name (num)
    (cond
	( (= 0 num) #\0 )
	( (= 1 num) #\1 )
	( (= 2 num) #\2 )
	( (= 3 num) #\3 )
	( (= 4 num) #\4 )
	( (= 5 num) #\5 )
	( (= 6 num) #\6 )
	( (= 7 num) #\7 )
	( (= 8 num) #\8 )
	( (= 9 num) #\9 )
	( t 'X )
    )
)

(defun get-name (num)
    (cond
	( (or (= 1 num) (= 2 num)) "rook" )
	( (or (= 3 num) (= 4 num)) "knight" )
	( (or (= 5 num) (= 6 num)) "bishop" )
	( (or (= 7 num) (= 8 num)) "guard" )
	( (or (= 9 num) (= 10 num)) "king" )
	( (or (= 11 num) (= 12 num)) "cannon" )
	( (or (= 13 num) (= 14 num)) "pawn" )
	( t "unknown" )
    )
)

(defun deploy-heuristic (lst &optional rn)
    ;(format t "deploy: ~S ~S~%" lst rn)
    (cond
	( (equal '((1 3) 13) lst) (open-rook-way) )
	( (equal '((7 3) 13) lst) (open-rook-way) )
	( (equal '((5 2) 3) lst) (bridge-cannon) )
	( (equal '((2 2) 3) lst) (bridge-cannon) )
	( (equal '((4 2) 11) lst) (front-cannon) )
	( (equal '((4 3) 5) lst) (defence-pawn) )
	( (eq 0 rn) (open-rook-way) )
	( (eq 1 rn) (bridge-cannon) )
	( (eq 2 rn) (front-cannon) )
	( (eq 3 rn) (front-bishop) )
	( t (deploy-heuristic nil (random 4)) )
    )
)

(defun open-rook-way ()
    (cond
	( (and (exist-piece 0 6)
	       (unreachable '(0 2) '((0 6) 2))
	  )
	      (append (human-move '((0 6) (1 6))) '(14))
        )
	( (and (exist-piece 8 6)
	       (unreachable '(8 2) '((8 6) 2))
	  )
	      (append (human-move '((8 6) (8 7))) '(14))
	)
	( t (deploy-heuristic nil (1+ (random 3))) )
    )
)

(defun bridge-cannon ()
    (cond
	( (not (exist-piece 5 7))
	    (append (human-move '((6 9) (5 7))) '(4))
	)
	( (not (exist-piece 2 7))
	    (append (human-move '((1 9) (2 7))) '(4))
	)
	( t (front-cannon) )
    )
)

(defun front-cannon ()
    (if (not (exist-piece 4 7))
	(cond
	    ( (reachable '(4 7) '((7 7) 12))
		(append (human-move '((7 7) (4 7))) '(12))
	    )
	    ( (reachable '(4 7) '((1 7) 12))
		(append (human-move '((2 7) (4 7))) '(12))
	    )
	    ( t (front-bishop) )
	)
	(front-bishop)
    )
)

(defun front-bishop ()
    (cond
	( (not (exist-piece 4 6))
	    (append (human-move '((2 9) (4 6))) '(6))
	)
	( (not (exist-piece 5 6))
	    (append (human-move '((7 9) (5 6))) '(6))
        )
	( t (defence-pawn) )
    )
)

(defun unreachable (pos apiece)
    (not (reachable pos apiece))
)

(defun reachable (pos apiece)
    (member pos (movable apiece) :test 'equal)
)

(defun defence-pawn ()
    (if (is-cannon (get-piece '(4 7)))
	(cond
	    ( (not (exist-piece 1 6))
		(append (human-move '((4 6) (3 6))) '(14))
	    )
	    ( (and (not (exist-piece 7 6))
		   (not (exist-piece 5 6))
	      )
		(append (human-move '((4 6) (5 6))) '(14))
	    )
	    ( t 
		(append (human-move '((6 6) (7 6))) '(14))
	    )
	)
    )
)

(defun terminate ()
    (if *quit* t nil)
)
