(in-package "USER")

(defun movable (lst)
    (let* ((piece (second lst))(bl (evenp piece)))
	(if bl (setf piece (1- piece)))
	(cond
	    ( (eq piece *r*) (r-movable (first lst) bl) )
	    ( (eq piece *n*) (n-movable (first lst) bl) )
	    ( (eq piece *b*) (b-movable (first lst) bl) )
	    ( (eq piece *g*) (g-movable (first lst) bl) )
	    ( (eq piece *k*) (k-movable (first lst) bl) )
	    ( (eq piece *c*) (c-movable (first lst) bl) )
	    ( (eq piece *p*) (p-movable (first lst) bl) )
	)
    )
)

(defun r-movable (lst b)
    (let ((result nil)(x (first lst))(y (second lst)))
	(append result
	        (scan (1+ x) to 8 b :y y)
		(scan (1- x) downto 0 b :y y)
		(scan (1+ y) to 9 b :x x)
		(scan (1- y) downto 0 b :x x)
	)
    )
)

(defmacro scan (start to end bool &key x y)
    (let ((piece (gensym))(i (gensym)))
	(cond
	    ( x
		`(loop for i from ,start ,to ,end
		    do (setf piece (aref *board* ,x i))
		    when (zerop piece)
			collect (list ,x i)
		    else when (eq ,bool (oddp piece))
			collect (list ,x i)
		    until (or (> piece 0) (> i 8) (< i 0))
		)
	    )
	    ( y
	        `(loop for i from ,start ,to ,end
	            do (setf piece (aref *board* i ,y))
		    when (zerop piece)
			collect (list i ,y)
		    else when (eq ,bool (oddp piece))
			collect (list i ,y)
		    until (or (> piece 0) (> i 7) (< i 0))
	        )
	    )
	    ( t (format t "No such form allowed in scan") nil)
	)
    )
)

(defun n-movable (lst b)
    (let ((result nil)(x (first lst))(y (second lst)))
        (append result
	        (knight-move (check-n-block x y) b)
        )
    )
)

(defun check-n-block (x y)
    (let ((result nil))
	(append result
		(blocked (1+ x) y :east t)
		(blocked (1- x) y :west t)
		(blocked x (1+ y) :north t)
		(blocked x (1- y) :south t)
	)
    )
)

(defun blocked (x y &key north south east west)
    (cond
	( (or (< x 0) (< y 0) (> x 8) (> y 9)) nil )
	( (exist-piece x y) nil)
	( t (cond
		(east (list (list *east* x y)) )
		(west (list (list *west* x y)) )
		(north (list (list *north* x y)) )
		(south (list (list *south* x y)) )
		(t (print "Illegal condition in block") nil)
	    )
	)
    )
)

(defun exist-piece (x y)
    (if (zerop (aref *board* x y))
	nil
	T
    )
)

(defun knight-move (lst b)
    (if (null lst)
	nil
        (let ( (d (first (first lst)))
	       (x (second (first lst)))
	       (y (third (first lst)))
	     )
	    (cond
	        ( (eq d *north*)
	          (append (enter (1- x) (1+ y) b)
			  (enter (1+ x) (1+ y) b)
			  (knight-move (rest lst) b)
		  )
		)
		( (eq d *south*)
		  (append (enter (1- x) (1- y) b)
			  (enter (1+ x) (1- y) b)
			  (knight-move (rest lst) b)
		  )
		)
		( (eq d *east*)
		  (append (enter (1+ x) (1- y) b)
			  (enter (1+ x) (1+ y) b)
			  (knight-move (rest lst) b)
		  )
		)
		( (eq d *west*)
		  (append (enter (1- x) (1- y) b)
			  (enter (1- x) (1+ y) b)
			  (knight-move (rest lst) b)
		  )
		)
		(t (print "Illegal condition in knight-move") nil)
	    )
	)
    )
)

(defun enter (x y bool &key cannon)
    (if (or (< x 0) (< y 0) (> x 8) (> y 9))
	nil
	(let ((piece (aref *board* x y)))
	    (cond
		( (zerop piece) (list (list x y)) )
		( cannon
		    (if (and (eq bool (oddp piece))
			     (not (is-cannon piece))
			)
			(list (list x y))
		    )
		)
		( (eq bool (oddp piece)) (list (list x y)) )
		( t nil )
	    )
	)
    )
)

(defun is-cannon (piece)
    (if (or (eq piece *c*) (eq (1- piece) *c*))
        T
        nil
    )
)

(defun b-movable (lst b)
    (let ((result nil)(x (first lst))(y (second lst)))
	(append result
	    (bishop-move (check-b-block x y) b)
        )
    )
)

(defun check-b-block (x y)
    (b-block (check-n-block x y))
)

(defun b-block (lst)
    (if (null lst)
	nil
	(let ( (d (first (first lst)))
	       (x (second (first lst)))
	       (y (third (first lst)))
	     )
	    (cond
		( (eq d *north*)
		  (append (blocked (1- x) (1+ y) :north t) ;northwest
			  (blocked (1+ x) (1+ y) :east t)  ;northeast
			  (b-block (rest lst))
		  )
		)
		( (eq d *east*)
		  (append (blocked (1+ x) (1+ y) :east t) ;northeast
			  (blocked (1+ x) (1- y) :south t);southeast
			  (b-block (rest lst))
		  )
		)
                ( (eq d *south*)
		  (append (blocked (1+ x) (1- y) :south t);southeast
			  (blocked (1- x) (1- y) :west t) ;southwest
                          (b-block (rest lst))
                  )
                )
		( (eq d *west*)
		  (append (blocked (1- x) (1- y) :west t) ;southwest
			  (blocked (1- x) (1+ y) :north t);northwest
			  (b-block (rest lst))
                  )
                )
	    )
	)
    )
)

(defun bishop-move (lst b)
    (if (null lst)
	nil
	(let ( (d (first (first lst)))
	       (x (second (first lst)))
	       (y (third (first lst)))
             )
	    (cond
		( (eq d *north*)	;actual direction is northwest
		  (append (enter (1- x) (1+ y) b)
			  (bishop-move (rest lst) b)
                  )
		)
		( (eq d *east*)		;actual direction is northeast
		  (append (enter (1+ x) (1+ y) b)
			  (bishop-move (rest lst) b)
                  )
		)
                ( (eq d *south*)	;actual direction is southeast
		  (append (enter (1+ x) (1- y) b)
			  (bishop-move (rest lst) b)
                  )
                )
		( (eq d *west*)		;actual direction is southwest
		  (append (enter (1- x) (1- y) b)
			  (bishop-move (rest lst) b)
                  )
                )
	    )
	)
    )
)

(defun g-movable (lst b)
    (let ((result nil)(x (first lst))(y (second lst)))
	(append result
	        (royal-move (check-royal x y) b)
	)
    )
)

(defun check-royal (x y)
    (cond
	( (or (< x 3) (> x 5) (and (> y 2) (< y 7))) nil )
	( (= 3 x)
	  (cond
	      ( (= 0 y) '((3 1)(4 1)(4 0)) )
	      ( (= 1 y) '((3 0)(3 2)(4 1)) )
	      ( (= 2 y) '((3 1)(4 1)(4 2)) )
	      ( (= 7 y) '((3 8)(4 8)(4 7)) )
	      ( (= 8 y) '((3 7)(3 9)(4 8)) )
	      ( (= 9 y) '((3 8)(4 8)(4 9)) )
	      ( t (print "Illegal co-ordinates in check-royal") nil )
	  )
	)
	( (= 4 x)
	  (cond
	      ( (= 0 y) '((4 1)(5 0)(3 0)) )
	      ( (= 1 y) '((4 2)(5 2)(5 1)(5 0)(4 0)(3 0)(3 1)(3 2)) )
	      ( (= 2 y) '((4 1)(3 2)(5 2)) )
	      ( (= 7 y) '((4 8)(5 7)(3 7)) )
	      ( (= 8 y) '((4 7)(3 7)(3 8)(3 9)(4 9)(5 9)(5 8)(5 7)) )
	      ( (= 9 y) '((4 8)(3 9)(5 9)) )
	      ( t (print "Illegal co-ordinates in check-royal") nil )
	  )
	)
	( (= 5 x)
	  (cond
	      ( (= 0 y) '((4 1)(5 1)(4 0)) )
	      ( (= 1 y) '((5 2)(5 0)(4 1)) )
	      ( (= 2 y) '((5 1)(4 1)(4 2)) )
	      ( (= 7 y) '((4 8)(5 8)(4 7)) )
	      ( (= 8 y) '((5 7)(4 8)(5 9)) )
	      ( (= 9 y) '((5 8)(4 8)(4 9)) )
	      ( t (print "Illegal co-ordinates in check-royal") nil )
	  )
	)
	( t (print "Illegal co-ordinates in check-royal") nil )
    )
)

(defun royal-move (lst bool)
    (if (null lst)
	nil
	(let ( (x (first (first lst)))
	       (y (second (first lst))) )
	    (append (enter x y bool)
		    (royal-move (rest lst) bool)
	    )
        )
    )
)

(defun k-movable (lst b)
    (g-movable lst b)
)

(defun c-movable (lst b)
    (let ((result nil)(x (first lst))(y (second lst)))
	(append result
		(cannon-move (get-over-piece x y) b)
	)
    )
)

(defun get-over-piece (x y)
    (let ((result nil))
	(append result
		(over-scan (1+ x) to 7 *east* :y y)
		(over-scan (1- x) downto 1 *west* :y y)
		(over-scan (1+ y) to 8 *north* :x x)
		(over-scan (1- y) downto 1 *south* :x x)
		(check-palace x y)
	)
    )
)

(defmacro over-scan (start to end d &key x y)
    (let ((piece (gensym))(i (gensym)))
	(cond
	    ( x
		`(loop for i from ,start ,to ,end
		    do (setf piece (aref *board* ,x i))
		    thereis (when (is-cannon piece) nil)
		    thereis (when (and (plusp piece)
				       (not (is-cannon piece)) )
				(list (list ,d ,x i))
			    )
		    finally nil
                )
            )
	    ( y
		`(loop for i from ,start ,to ,end
		    do (setf piece (aref *board* i ,y))
		    thereis (when (is-cannon piece) nil)
		    thereis (when (and (plusp piece)
				       (not (is-cannon piece)) )
				(list (list ,d i ,y))
			    )
		    finally nil
                )
            )
	    ( t (format t "No such form allowed in over-scan") nil)
        )
    )
)

(defun check-palace (x y)
    (let ((throne nil))
        (if (< y 3)
	    (setf throne (plusp (aref *board* 4 1)))
	    (if (> y 6)
		(setf throne (plusp (aref *board* 4 8)))
	    )
	)
        (cond
	    ( (and (> y 2) (< y 7)) nil) 
	    ( (< x 3) nil )
	    ( (> x 5) nil )
	    ( (= 3 x)
	      (cond
	          ( (= 0 y) (get-diagonal throne 5 2) )
		  ( (= 2 y) (get-diagonal throne 5 0) )
                  ( (= 7 y) (get-diagonal throne 5 9) )
		  ( (= 9 y) (get-diagonal throne 5 7) )
		  ( t nil )
              )
	    )
	    ( (= 5 x)
	      (cond
		  ( (= 0 y) (get-diagonal throne 3 2) )
                  ( (= 2 y) (get-diagonal throne 3 0) )
                  ( (= 7 y) (get-diagonal throne 3 9) )
                  ( (= 9 y) (get-diagonal throne 3 7) )
		  ( t nil )
	      )
	    )
	    ( t nil )
	)
    )
)

(defun get-diagonal (bool x y)
    (if bool
	(list (list *diagonal* x y))
	nil
    )
)

(defun cannon-move (lst b)
    (if (null lst)
	nil
	(let ( (d (first (first lst)))
	       (x (second (first lst)))
	       (y (third (first lst)))
	     )
	    (cond
		( (eq d *north*)
		  (append (c-scan (1+ y) to 9 b :x x)
			  (cannon-move (rest lst) b)
		  )
		)
		( (eq d *east*)
		  (append (c-scan (1+ x) to 8 b :y y)
			  (cannon-move (rest lst) b)
		  )
		)
		( (eq d *west*)
		  (append (c-scan (1- x) downto 0 b :y y)
			  (cannon-move (rest lst) b)
		  )
		)
		( (eq d *south*)
		  (append (c-scan (1- y) downto 0 b :x x)
			  (cannon-move (rest lst) b)
		  )
		)
		( (eq d *diagonal*)
		  (append (list (list x y))
			  (cannon-move (rest lst) b)
		  )
		)
	    )
	)
    )
)

(defmacro c-scan (start to end bool &key x y)
    (let ((piece (gensym))(i (gensym)))
	(cond
	    ( x
	        `(loop for i from ,start ,to ,end
		    do (setf piece (aref *board* x i))
		    when (zerop piece)
		        collect (list x i)
		    else when (and (eq ,bool (oddp piece))
			           (not (is-cannon piece)) )
			     collect (list x i)
		    until (or (plusp piece) (> i 8) (minusp i))
		)
	    )
	    ( y
	        `(loop for i from ,start ,to ,end
                    do (setf piece (aref *board* i y))
		    when (zerop piece)
			collect (list i y)
		    else when (and (eq ,bool (oddp piece))
				   (not (is-cannon piece)) )
			     collect (list i y)
		    until (or (plusp piece) (> i 7) (minusp i))
		)
	    )
	)
    )
)

(defun p-movable (lst b)
    (let ((result nil)(x (first lst))(y (second lst)))
        (append result
		(if b
		    (enter x (1- y) b)
		    (enter x (1+ y) b)
		)
		(enter (1+ x) y b)
		(enter (1- x) y b)
	)
    )
)
