(in-package "USER")

(defun make-tree (root children)
    (cons root children)
)

(defun make-leaf (node)
    (make-tree node nil)
)

(defun tree-parent (tree)
    (first tree)
)

(defun tree-children (tree)
    (rest tree)
)

(defun leafp (tree)
    (null (tree-children tree))
)

(defun opponent (player)
    (if (eq *blue* player)
	*red*
	*blue*
    )
)

; pairs : '( ((nil) ((x y) (x y))) ((nil) ((x y) (x y)) ... ) ... )
(defun grow-tree (pairs depth player) 
    (if (= 0 depth)
	(make-leaf pairs)
	(make-tree pairs
	    (mapcar #'(lambda (child)
			  (grow-tree child (1- depth)
				     (opponent player))
		      )
	        (next-move pairs player)
	    )
	)
    )
)

(defun next-move (pairs player)
    (let (result (pair (first pairs)))
	(if (null pair)
	    nil
	    (if (and (null (numberp (first pair)))
		     (null (rest pairs))
		)
		(possible-next-move pairs player)
		(progn
	            (change-board pairs)
		    (setf result (possible-next-move  pairs player))
		    (change-board pairs :rev t)
		    ;(show)
		    result
		)
	    )
	)
    )
)

(defun change-board (pairs &key rev)
    (let ((pair (first pairs)))
	(if (null pairs)
	    nil
	    (progn 
	        (if (listp (first pair))
		    (if rev (human-move pair :r t :pseudo t)
			    (human-move pair :pseudo t)
		    )
		    (if rev (change-board (reverse (rest pairs)) :rev t)
			    (change-board (rest pairs))
		    )
	        )
		(if (listp (first pair))
	            (if rev (change-board (rest pairs) :rev t)
		            (change-board (rest pairs))
		    )
		)
	    )
	)
    )
)

(defun possible-next-move (pairs player)
    (let ((pieces nil))
	(if (equal *blue* player)
	    (setf pieces (blue-pieces))
	    (setf pieces (red-pieces))
	)
	;(format t "pieces:~S*****~%" pieces)
	(possible-next pairs pieces)
    )
)

; pieces : '(((x y) p) ((x y) p) ...)
(defun possible-next (pairs pieces)
    (let ((piece (first pieces)))
	;(format t "~S ~S~%" pairs piece)
	(if (null piece)
	    nil
	    (let ((next-lst (movable piece)))
		(append 
		    (get-pairs pairs (first piece) next-lst)
		    (possible-next pairs (rest pieces))
		)
	    )
        )
    )
)

(defun get-pairs (pairs start lst)
    (if (null lst)
	nil
	(append (list (append pairs (list (list start (first lst)))))
		(get-pairs pairs start (rest lst))
	)
    )
)

(setf *level* 0)

(defun minimax (tree lev)
(let (flag)
    (if (evenp lev)
	(setf flag nil)
	(setf flag t)
    )
    (if (leafp tree)
	(values (get-rank (tree-parent tree))
		(tree-parent tree)
	)
	(let* ((child-ranks
	          (mapcar #'(lambda (subtree) (minimax subtree (1- lev)))
			  (tree-children tree)
		  )
		)
		(best-rank 
		    (apply 
			(if flag #'min #'max)
			 child-ranks
		    )
		)
	       )
	    ;(format t "****best-rank(lev~S):~S~%" lev best-rank)
	    (values best-rank
		(tree-parent (elt (tree-children tree)
				  (position best-rank child-ranks)
			     )
	        )
	    )
	)
    )
))

(defun get-rank (lst)
    (if (rest lst)
	(let (result)
	    (change-board lst)
	    (setf result (score))
	    ;(format t "xxxxxxxxxxxxxxxxxxxxx~S~%" lst)
	    ;(format t "xxx ~S xxx~%" result)
	    ;(show)
	    (change-board lst :rev t)
	    ;(format t "**********************~%")
	    ;(show)
	    result
	)
	0
    )
)
