(IN-PACKAGE 'USER)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; function SIMPLIFY
;;   that takes an arithmetic expression and returns a new one 
;;   in which the following improvements are made:
;;   (a) any subexpression consisting of the function * followed
;;   by a list of arguments, one of which is 0, is replaced by 0;
;;   (b) any occurrence of 1 as an argument to * is eliminated
;;   (c) any occurrence of 0 as an argument to + is eliminated, and
;;   if only one argument remains, the occurrence of + is eliminated.
;;

(defun simplify (lst)
  "Simplify the given arithmetic expression list."
  (helper
    (helper (if (listp lst) (mapcar #'simplify lst) (helper lst)) )
  )
)

(defun helper (lst)
  (cond
    ((atom lst) lst)
    ((equal (first lst) '*)
     (cond
       ((member '0 (rest lst)) '0)
       ((member '1 (rest lst))
	(remove '1 lst)
       )
       ((equal (length (rest lst)) 1) (cadr lst))
       (t lst)
     )
    )
    ((equal (first lst) '+)
     (cond
       ((member '0 (rest lst)) (remove '0 lst))
       ((equal (length (rest lst)) 1) (cadr lst))
       (t lst)
     )
    )
    (t "Illegal expression")
  )
)
