CREATE OR REPLACE PACKAGE phone_pkg AS

    TYPE person_rec_type IS RECORD              -- person-info record
       (person_id       NUMBER(6),
        last_name       VARCHAR2(15),
        first_name      VARCHAR2(15),
        title           VARCHAR2(8),
        user_name       VARCHAR2(8),
        address         VARCHAR2(12),
        url             VARCHAR2(20));

    TYPE phone_rec_type IS RECORD               -- phone-list record
       (person_id       NUMBER(6),
        phone_no        NUMBER(10),
        phone_type      VARCHAR2(10));

    TYPE mix_rec_type IS RECORD
       (person_id       NUMBER(6),
        first_name      VARCHAR2(15),
        last_name       VARCHAR2(15),
        phone_no        NUMBER(10),
        phone_type      VARCHAR2(10));

    PROCEDURE get_phone_by_last_name(lname IN VARCHAR2);

    PROCEDURE display_mix_rec(mix_rec IN mix_rec_type);

END phone_pkg;
/
CREATE OR REPLACE PACKAGE BODY phone_pkg AS

    PROCEDURE get_phone_by_last_name(lname IN VARCHAR2)
    IS
        CURSOR mix_cur(lname IN VARCHAR2)
        IS
            SELECT a.person_id, first_name, last_name,
                   phone_no, phone_type
            FROM person_info_table a, phone_list_table b
            WHERE (lname IS NULL OR 
                   last_name LIKE ('%' || LOWER(lname) || '%')) AND
                  (a.person_id = b.person_id);

        mix_rec   mix_rec_type;

    BEGIN

        IF NOT mix_cur%ISOPEN THEN          -- open the cursor if
            OPEN mix_cur(lname);             -- not open yet
        END IF;

        -- Fetch rows into PL/SQL table

        LOOP
            FETCH mix_cur INTO mix_rec;
            EXIT WHEN mix_cur%NOTFOUND;
            display_mix_rec(mix_rec);
        END LOOP;

        CLOSE mix_cur;

    END get_phone_by_last_name;

    PROCEDURE display_mix_rec(mix_rec IN mix_rec_type)
    IS
    BEGIN
        DBMS_OUTPUT.PUT_LINE
           (mix_rec.person_id || ' | ' || 
            INITCAP(mix_rec.first_name) || ' | ' || 
            INITCAP(mix_rec.last_name) || ' | ' ||
            mix_rec.phone_no || ' | ' || 
            INITCAP(mix_rec.phone_type));
    END display_mix_rec;


END phone_pkg;
/
