#! /usr/local/bin/perl
 
#############################################################################
# Search.pl
# This perl program is run by selecting menu of 'Search.html'. When 'Show
# all the lists' button is clicked, this program generates an HTML which
# shows all the lists of database as appropriate form. When a user fills
# the search text field, selects the item, and clicks 'Search now' button
# on the 'Search.html', this perl program searches for the item and shows
# the result as an HTML form.
# A name can be searched with first name, last name, first and last name,
# or full name with middle name. If the name field of database has only
# one - first or last - name, then this program compares with both of input
# first and last name and shows the list if one of them corresponded.
# A phone number can be searched with ddddddd, ddd dddd, ddd-dddd, Ext.dddd,
# or x.dddd. User input and database phone number can be one of above forms.
# (d : digit )
#
# @author: Jungkee (Jake) Kim
#
 
#----------------------------------------------------------------------------
# Main Function
#

&PrintHeader;
&ReadEntry;
&ProcessData;

#----------------------------------------------------------------------------
# Subroutines
#
 
#------------------------------------------
# PrintHeader
# This subroutine returns the magic line.
# Magic line informs to WWW that this is an HTML document.
 
sub PrintHeader {
    print "Content-type: text/html\n\n";
}
 
#------------------------------------------
# ReadEntry
# Read entry from the user adding form.
 
sub ReadEntry {
    &ReadParse;
    $string = $in{'searchitem'};
    $select = $in{'item'};
    return 1;
}

#------------------------------------------
# ReadParse
# This subroutine reads the arguments passed from the fill-out form
 
sub ReadParse {
    local ($i, $loc, $key, $val);

    # Note the added else-block for command line debugging:
    if ($ENV{'REQUEST_METHOD'} eq "GET") {		# Read in text
        $in = $ENV{'QUERY_STRING'};
    }
    elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
        for ($i = 0; $i < $ENV{'CONTENT_LENGTH'}; $i++) {
            $in .=getc;
        }
    }
    else {  # can be used for debugging on command line
        $in = <STDIN>;
        print "$in\n";
    }

    # Split input string into an array of strings:
    @in = split(/&/,$in);

    # Process each array element individually:
    foreach $i (0 .. $#in) {
        $in[$i] =~ s/\+/ /g;		# Convert plus signs to spaces
        $in[$i] =~ s/%(..)/pack("c",hex($1))/ge;
                                # Convert %XX from hex numbers to alphanumeric
        $loc = index($in[$i], "=");		# Split onto key and value.
        $key = substr($in[$i],0,$loc);		# Extract the key
        $val = substr($in[$i],$loc+1);		# Extract the value
        $in{$key} .= '\0' if (defined($in{$key}));
						#\0 is the multiple separator
        $in{$key} .= $val;
    }
    return 1;
}

#------------------------------------------
# ProcessData
# This subroutine determines which button was pressed and acts accordingly
# beween two submit buttons on the HTML form.

sub ProcessData {
    if ( defined($in{'show'}) ) {
        &ShowLists;
    }
    elsif ( defined($in{'search'}) ) {
        &SearchLists;
    }
    else {}
    return 1;
}

#------------------------------------------
# ShowLists
# This subroutine makes an HTML page which displays the entire lists of
# database.

sub ShowLists {
    # HTML opening prints
    print "<HTML>\n";
    print "<HEAD>\n";
    print "<TITLE>Show all the Faculty list</TITLE>\n";
    print "</HEAD>\n";
    print "<BODY>\n\n";
    print "<H1>Current input Faculty list</H1>\n";

    # Open and Read the flat file database and write the output
    open(DATAIN, "faculty.db");
    flock(DATAIN, 2);

    while ( $record = <DATAIN> ) {
        &GetARecord;
        &PrintARecord;
    }

    print "</BODY>\n";
    print "</HTML>";
    return 1;
}

#------------------------------------------
# GetARecord
# Get each fields of a record which has been read from flat file.

sub GetARecord {
    chop($record);
    $recordlength = length( $record );
    $position = index($record, "/");
    $name = substr($record, 0, $position);
    $record = substr($record, $position + 1, $recordlength - $position);
    $position = index($record, "/");
    $phone = substr($record, 0, $position);
    $record = substr($record, $position + 1, $recordlength - $position);
    $position = index($record, "/");
    $room = substr($record, 0, $position);
    $email = substr($record, $position + 1);
}

#------------------------------------------
# PrintARecord
# Print information of a record as HTML form.

sub PrintARecord {
    print "<UL>\n";
    print "<LI><B>Name:</B> $name</LI>\n";
    print "<UL>\n";
    print "<LI><B>Phone:</B> $phone</LI>\n";
    print "<LI><B>Room:</B> $room</LI>\n";
    print "<LI><B>Email:</B> <A HREF=\"mailto:$email\">$email</A></LI>\n";
    print "</UL></UL>\n";
}

#------------------------------------------
# SearchLists
# This subroutine searches matched key word in the flat file database.
# The key word is fetched from HTML form and used as name or phone number.
# The kind of key is determined by the input select button 'item'.

sub SearchLists {
    # HTML opening prints
    print "<HTML>\n";
    print "<HEAD>\n";
    print "<TITLE>Search records with a key word</TITLE>\n";
    print "</HEAD>\n";
    print "<BODY>\n\n";
    print "<H1>The result of search</H1>\n";
    print "<B>Search item";

    # Open and Read the flat file database and write the output
    open(DATAIN, "faculty.db");
    flock(DATAIN, 2);

    if ($select eq 'name') {
        print " by name:</B> \"$string\"<BR>\n";
        &SearchName;
    }
    elsif ($select eq 'phone') {
        print " by phone number:</B> \"$string\"<BR>\n";
        &SearchPhone;
    }
    else {}

    print "</BODY>\n";
    print "</HTML>";
    return 1;
}

#------------------------------------------
# SearchName
# Searches by name.
# $Found : when the name is found, 1 or 0.

sub SearchName {
    local ($Found) = @_;

    $Found = 0;
    while ( $record = <DATAIN> ) {
        &GetARecord;
	$IsMatch = 0;
	&NameAnalysis;
	if ( $IsMatch == 1 ) {
	    &PrintARecord;
            $Found = 1;
	}
    }
    if ( $Found == 0 ) {
        print "<H3>No such data can be found.</H3>\n";
    }
    return 1;
}

#------------------------------------------
# NameAnalysis
# Analyzes the each name and compares. If the name is matched,
# the variable, $IsMatch will be set as 1.
# Possible name form of user input and name field of database:
#    first_name,
#    last_name,
#    first_name last_name,
#    first_name middle_name last_name.
# $IsNameFullname : Is the name from flat file full name ?
#                   0 - just one name.
#                   1 - first and last name.
#                   2 - first, middle and last name.
# $IsStrFullname : Is the name from input full name ?
#                  the possible value is as in $IsNameFullname.

sub NameAnalysis {
    $stringlength = length($string);
    $namelength = length($name);
    #if the length of input name and database name, then just compares name.
    if ( $stringlength == $namelength ) {
        if ($string =~ /$name/i) {
            $IsMatch = 1;
        }
        else {
             $IsMatch = 0;
        }
    }
    else {
        &ParseName;				#parse name from flat file
        &ParseStrName;				#parse name from HTML form
        if ( $IsStrFullname == 0 ) {
            if ( $IsNameFullname == 0 ) {
		$IsMatch = 0;		#both are one name and different length
	    }
	    else {			#only input name is one name
		if ( $stringlength == $namefirstlength && 
                     $string =~ /$firstname/i ) {
		    $IsMatch = 1;	#matches with first name
		}
                elsif ( $stringlength == $namelastlength &&
                        $string =~ /$lastname/i ) {
		    $IsMatch = 1;	#matches with last name
		}
		else {
		    $IsMatch = 0;
		}
	    }
	}
	else {				#input name is full name
	    #if name from flat file is unique name
	    if ( $IsNameFullname == 0 ) {
                if ( $strfirstlength == $namelength &&
                     $strfirst =~ /$name/i ) {
		    $IsMatch = 1;
		}
                elsif ( $strlastlength == $namelength &&
                        $strlast =~ /$name/i ) {
		    $IsMatch = 1;
		}
		else {
		    $IsMatch = 0;
		}
	    }
	    #In the case of both name is a kind of full name,
	    #just compares the match of first and last name.
	    #This routine ignores the middle name matching.
	    else {
                if ( $strfirstlength == $namefirstlength &&
                     $strfirst =~ /$firstname/i ) {
                    if ( $strlastlength == $namelastlength &&
                         $strlast =~ /$lastname/i ) {
		        $IsMatch = 1;
		    }
		    else {
			$IsMatch = 0;
		    }
		}
		else {
		    $IsMatch = 0;
		}
	    }
	}
    }
    return 1;
}

#------------------------------------------
# ParseName
# This routine analysises the name from flat file.
# If the name is unique name then set $IsNameFullname as 0.
# If the name is first and last name then set it as 1.
# If the name is first, middle and last name then set it as 2.

sub ParseName {
    local($position, $templength) = @_;
    local($temp) = '';

    $position = index($name, " ");
    #If no space in the name then it can be ascribed as unique name.
    if ($position == -1) {
        $IsNameFullname = 0;
        return 1;
    }
    $firstname = substr($name, 0, $position);
    $namefirstlength = length($firstname);
    $temp = substr($name, $position + 1, $namelength - $position);
    $templength = length($temp);
    $position = index($temp, " ");
    if ($position == -1) {
        $lastname = $temp;
        $namelastlength = length($lastname);
        $IsNameFullname = 1;
        return 1;
    }
    $middlename = substr($temp, 0, $position);
    $namemiddlelength = length($middlename);
    $lastname = substr($temp, $position + 1, $templength - $position);
    $namelastlength = length($lastname);
    $IsNameFullname = 2;
    return 1;
}

#------------------------------------------
# ParseStrName
# This routine analysises the name from input HTML form.
# The variable $IsStrFullname is set as in subroutine ParseName.

sub ParseStrName {
    local($position, $templength) = @_;
    local($temp) = '';

    $position = index($string, " ");
    if ($position == -1) {
        $IsStrFullname = 0;
        return 1;
    }
    $strfirst = substr($string, 0, $position);
    $strfirstlength = length($strfirst);
    $temp = substr($string, $position + 1, $stringlength - $position);
    $templength = length($temp);
    $position = index($temp, " ");
    if ($position == -1) {
        $strlast = $temp;
        $strlastlength = length($strlast);
        $IsStrFullname = 1;
        return 1;
    }
    $strmiddle = substr($temp, 0, $position);
    $strmiddlelength = length($strmiddle);
    $strlast = substr($temp, $position + 1, $templength - $position);
    $strlastlength = length($strlast);
    $IsStrFullname = 2;
    return 1;
}

#------------------------------------------
# SearchPhone
# Searches by phone number.
# The variable $Found is as in subroutine SearchName.
# $Error variable set 1 if phone number is not appropriate.

sub SearchPhone {
    local ($Found) = @_;

    $Found = 0;
    $Error = 0;
    while ( $record = <DATAIN> ) {
        &GetARecord;
	$IsMatch = 0;
        &PhoneAnalysis;
        exit 0 if $Error == 1;
	if ( $IsMatch == 1 ) {
            $Found = 1;
	    &PrintARecord;
	}
    }
    if ( $Found == 0 ) {
        print "<H3>No such data can be found.</H3>\n";
    }
    return 1;
}

#------------------------------------------
# PhoneAnalysis
# Analysises the phone numbers and converts them as 7 digit number.
# Compares both numbers and set $IsMatch as 1 when they are the same.

sub PhoneAnalysis {
    &GetInputPhone;
    &GetFieldPhone;
    if ( $InputPhone == $FieldPhone ) {
        $IsMatch = 1;
    }
    return 1;
}

#------------------------------------------
# GetInputPhone
# Gets the 7 digit phone number from input string.

sub GetInputPhone {
    $ReadPhone = $string;
    &GetPhoneNumber;
    $InputPhone = $PhoneNumber;
    return 1;
}

#------------------------------------------
# GetFieldPhone
# Gets the 7 digit phone number from name field of flat file.

sub GetFieldPhone {
    $ReadPhone = $phone;
    &GetPhoneNumber;
    $FieldPhone = $PhoneNumber;
    return 1;
}

#------------------------------------------
# GetPhoneNumber
# Gets the 7 digit phone number from $ReadPhone variable.
# This subroutine detaches alphabet and special letters and add '443'
# if 'Ext.' or 'x.' was in the variable, $ReadPhone.
# In the case of some error phone number, it shows error message and
# makes the main routine quit.

sub GetPhoneNumber {
    local($xposition, $position, $lastlength) = @_;
    local($temp) = '';
    $PhoneNumber = '';
    $xposition = index($ReadPhone, "x");
    #if no extension mark is found
    if ( $xposition == -1 ) {
        $phonelength = length($ReadPhone);

	#if phone number length is 7, all letters should be number
        if ( $phonelength == 7 ) {
            $position = index($ReadPhone, "\D");
            if ( $position == -1 ) {
                $PhoneNumber = $ReadPhone;
                return 1;
    	    }
	    #if the phone number has none digit number then error
	    else {
	        &PhoneError;
	        return 0;
	    }
        }

        $position = index($ReadPhone, " ");

        if ( $position == -1 ) {
            $position = index($ReadPhone, "-");
	}
	#no found '-' or phone number length is not 8
	if ( $position == -1 || $phonelength != 8 ) {
	    &PhoneError;
            return 0;
        }

	#if '-' is at the proper location
	if ( $position == 3 ) {
	    $firstdigit = substr($ReadPhone, 0, 3);
	    if ( $firstdigit =~ /\D/ ) {
		&PhoneError;
		return 0;
	    }
	    $lastdigit = substr($ReadPhone, 4);
            $lastlength = length($lastdigit);
	    if ( $lastdigit =~ /\D/ || $lastlength != 4) {
	        &PhoneError;
	        return 0;
	    }
	    else {
	        $PhoneNumber = $firstdigit . $lastdigit;
	        return 1;
	    }
	}
	else {
	    &PhoneError;
	    return 0;
	}
    }
    #if phone number has 'x' letter
    else {
	$position = index($ReadPhone, ".");
	$lastdigit = substr($ReadPhone, $position + 1);
	$lastlength = length($lastdigit);
	#If phone number has non digit letter or length is not 4,
	# then it must be error.
	if ( $lastdigit =~ /\D/ || $lastlength != 4) {
	    &PhoneError;
	    return 0;
	}
	#If phone number is right, return last 4 digits with '443'.
	else {
	    $firstdigit = '443';
	    $PhoneNumber = $firstdigit . $lastdigit;
	    return 1;
	}
    }
    return 0;
}

#------------------------------------------
# PhoneError
# This subroutine shows that the phonenumber is not proper form.

sub PhoneError {
    $Error = 1;
    print "Phone Number \"$ReadPhone\" is illegal!<BR>\n";
    return 1;
}
