#! /usr/local/bin/perl
#############################################################################
#  
#   File:    database.pl
#
#   Author:  Tom Scavo <trscavo@syr.edu>
#
#   Input:   data from the HTML form database.html
#
#   Output:  a Web page containing the search results
#
#   From the HTML page database.html, the user may choose to list the
#   complete database or search the database for a specific search string.
#
#############################################################################
#  
#   MAIN ROUTINES
#
#############################################################################

&ReadData;
&ProcessData;

#############################################################################
#  
#   SUBROUTINES
#
#############################################################################

sub ReadData {
	&ReadParse;
	$searchString = $in{'searchString'};
	return 1;
}

# Read and parse the data passed from fill-out form:
sub ReadParse {
	local ($i, $loc, $key, $val);
	
	# Note the added else-block for command line debugging:
	if ( $ENV{'REQUEST_METHOD'} eq "GET" ) {
		$in = $ENV{'QUERY_STRING'};
	} elsif ( $ENV{'REQUEST_METHOD'} eq "POST" ) {
		for ( $i = 0; $i < $ENV{'CONTENT_LENGTH'}; $i++ ) {
			$in .= getc;
		}
	} else {
		$in = <STDIN>;
		print "$in\n";
	}
	
	# Split input string into an array of strings:
	@in = split(/&/,$in);
	
	# Process each array element individually:
	foreach $i ( 0 .. $#in ) {
		# Convert plus signs to spaces:
		$in[$i] =~ s/\+/ /g;
		# Convert %nn from hex to alphanumeric:
		$in[$i] =~ s/%(..)/pack("c",hex($1))/ge;
		# Determine the location of the equals sign:
		$loc = index($in[$i], "=");
		# Extract the key:
		$key = substr($in[$i],0,$loc);
		# Extract the value:
		$val = substr($in[$i],$loc+1);
		# Multiple values are separated by a null character:
		$in{$key} .= '\0' if ( defined($in{$key}) );
		$in{$key} .= $val;
	}
	
	return 1; 
}

# On the HTML form, there are submit buttons called 'browse' and 'search'.
# This routine determines which button was pressed and acts accordingly.
# If the user presses the 'search' button, but forgets to type a search
# string, this routine simply reloads the HTML page.
sub ProcessData {
	if ( defined($in{'browse'}) ) {
		&BrowseDatabase;
	} elsif ( defined($in{'search'}) ) {
		if ( $searchString ) {
			&SearchDatabase;
		} else {
		        print 
'Location: http://osprey7.npac.syr.edu:3768/users-docs/njm/trscavo/database.html';
			print "\n\n";
		}
	} else {
		# can't happen!
	}
	
	return 1;
}

# Construct an HTML page that displays the entire database:
sub BrowseDatabase {
	print "Content-type: text/html\n\n";

	# Opening HTML statements:
	print "<HTML>\n";
	print "<HEAD>\n";
	print "<TITLE>Alpha LaTeX Database</TITLE>\n";
	print "</HEAD>\n";
	print "<BODY>\n\n";
	
	# Begin HTML body:
	print "<H1>Alpha LaTeX Registered Users</H1>\n\n";
	print "<OL>\n";

	# Read the database and write the output:
	open (DBFILE, "register.db");
        flock(DBFILE, 2);
	while ( $record = <DBFILE> ) {
		chop($record);
		if ( $record eq "<record>" ) {
			# pre-process this record:
			# (no pre-processing at this time)
		} elsif ( $record eq "</record>" ) {
			# post-process this record:
			$name = $foo{'name'};
			$email = $foo{'email'};
			$platform = $foo{'platform'};
			$TeXapp = $foo{'TeXapp'};
			print "  <LI>";
	print "$name  \n  &lt;<A HREF=\"mailto:$email\">$email</A>&gt; <BR>\n";
			print "  platform = $platform; TeXapp = $TeXapp";
			print " <P>\n";
		} else {
			# process this record:
			while ( $record =~ /(\w*): ([^;]*);/ ) {
				$foo{$1} = $2;
				$record = $';
			}
		}
	}
	flock (DBFILE, 8);
	close (DBFILE);
	
	# End HTML body:
	print "</OL>\n";
	
	# Closing HTML statements:
	print "\n</BODY>\n";
	print "</HTML>\n";
	
	return 1;
}

# Construct an HTML page that displays a portion of the database:
sub SearchDatabase {
	print "Content-type: text/html\n\n";

	# Opening HTML statements:
	print "<HTML>\n";
	print "<HEAD>\n";
	print "<TITLE>Alpha LaTeX Database</TITLE>\n";
	print "</HEAD>\n";
	print "<BODY>\n\n";
	
	# Begin HTML body:
	print "<H1>Alpha LaTeX Registered Users</H1>\n\n";
	print "searchString = $searchString\n\n";
	print "<OL>\n";
	$noMatch = 1;

	# Read the database and write the output:
	open (DBFILE, "register.db");
	flock(DBFILE, 2);

	while ( $record = <DBFILE> ) {
		chop($record);
		if ( $record eq "<record>" ) {
			# pre-process this record:
		} elsif ( $record eq "</record>" ) {
			# post-process this record:
			$isMatch = 0;
			foreach $key ( keys(%foo) ) {
				if ( $foo{$key} =~ /$searchString/i ) {
					$isMatch = 1;
					last;
				}
			}
			if ( $isMatch ) {
				$noMatch = 0;
				$name = $foo{'name'};
				$email = $foo{'email'};
				$platform = $foo{'platform'};
				$TeXapp = $foo{'TeXapp'};
				print "  <LI>";
	print "$name  \n  &lt;<A HREF=\"mailto:$email\">$email</A>&gt; <BR>\n";
	print "  platform = $platform; TeXapp = $TeXapp";
				print " <P>\n";
			}
		} else {
			# process this record:
			while ( $record =~ /(\w*): ([^;]*);/ ) {
				$foo{$1} = $2;
				$record = $';
			}
		}
	}
	flock (DBFILE, 8);
	close (DBFILE);
	
	# End HTML body:
	if ( $noMatch ) {
		print "No matches found!\n";
	}
	print "</OL>\n";
	
	# Closing HTML statements:
	print "\n</BODY>\n";
	print "</HTML>\n";
	
	return 1;
}

