#! /usr/local/bin/perl
#############################################################################
#  
#   File:    register.pl
#
#   Author:  Tom Scavo <trscavo@syr.edu>
#
#   Input:   data from the HTML form generated by validate.pl
#
#   Output:  depends on the input data (see below)
#
#   This script processes the HTML form generated by validate.pl.
#   If the user presses the "Yes" button, the script writes an entry
#   in the database and loads a dummy HTML page.  (The production
#   version of this script would load a more detailed page with links
#   to download the requested software.)  If the user presses the "No"
#   button, the script reloads the original form register.html.
#
#############################################################################
#  
#   MAIN ROUTINES
#
#############################################################################

&ReadData;
if ( $install eq "Yes" ) {
	&ProcessData;	
} elsif ( $install eq "No" ) {
	&ReloadForm;
} else {
	# This can't happen!
}

#############################################################################
#  
#   SUBROUTINES
#
#############################################################################

sub ReadData {
	&ReadParse;
	$name = $in{'name'};
	$email = $in{'email'};
	$platform = $in{'platform'};
	$TeXapp = $in{'TeXapp'};
	$otherApp = $in{'otherApp'};
	$install = $in{'install'};
	
	return 1;
}

# Read and parse the data passed from fill-out form:
sub ReadParse {
	local ($i, $loc, $key, $val);
	
	# Note the added else-block for command line debugging:
	if ( $ENV{'REQUEST_METHOD'} eq "GET" ) {
		$in = $ENV{'QUERY_STRING'};
	} elsif ( $ENV{'REQUEST_METHOD'} eq "POST" ) {
		for ( $i = 0; $i < $ENV{'CONTENT_LENGTH'}; $i++ ) {
			$in .= getc;
		}
	} else {
		$in = <STDIN>;
		print "$in\n";
	}
	
	# Split input string into an array of strings:
	@in = split(/&/,$in);
	
	# Process each array element individually:
	foreach $i ( 0 .. $#in ) {
		# Convert plus signs to spaces:
		$in[$i] =~ s/\+/ /g;
		# Convert %nn from hex to alphanumeric:
		$in[$i] =~ s/%(..)/pack("c",hex($1))/ge;
		# Determine the location of the equals sign:
		$loc = index($in[$i], "=");
		# Extract the key:
		$key = substr($in[$i],0,$loc);
		# Extract the value:
		$val = substr($in[$i],$loc+1);
		# Multiple values are separated by a null character:
		$in{$key} .= '\0' if ( defined($in{$key}) );
		$in{$key} .= $val;
	}
	
	return 1; 
}

# Print a dummy HTML page with a link to a piece of software (the
# production version of this page would provide links to the
# *requested* pieces of software):
sub ProcessData {
	
	# Update the database:
	&UpdateDBFile;
	
	# Print an HTML page:
	print "Content-type: text/html\n\n";
	
	# Opening HTML statements:
	print "<HTML>\n";
	print "<HEAD>\n";
	print "<TITLE>Download Alpha LaTeX</TITLE>\n";
	print "</HEAD>\n";
	print "<BODY>\n\n";
	
	# Begin HTML body:
	print '<A HREF="ftp://www.cs.umd.edu/pub/faculty/keleher/Alpha/latex_guide.sit.hqx"><EM>Download</EM></A>';
	print "\n";
	
	# Closing HTML statements:
	print "\n</BODY>\n";
	print "</HTML>\n";
		
	return 1;
}

# Update the database with the input data:
sub UpdateDBFile {
	open (DBFile, ">>register.db") || die("error: can not open register.db");
	flock (DBFile, 2);
	print DBFile "<record>\n";
	print DBFile "name: $name; email: $email;\n";
	print DBFile "platform: $platform; "; 
	if ( $TeXapp eq "other" ) {
		print DBFile "TeXapp: $otherApp;\n";
	} else {
		print DBFile "TeXapp: $TeXapp;\n";
	}
	print DBFile "</record>\n";
	flock (DBFile, 8);
	close (DBFile);
		
	return 1;
}

# Reload the original HTML form:
sub ReloadForm {
print 
'Location: http://osprey7.npac.syr.edu:3768/users-docs/njm/trscavo/register.html';
	print "\n\n";
		
	return 1;
}

