#! /usr/local/bin/perl
#############################################################################
#  
#   File:    validate.pl
#
#   Author:  Tom Scavo <trscavo@syr.edu>
#
#   Input:   data from the HTML form register.html
#
#   Output:  a confirmation page containing the input data *or*
#            a page of error messages
#
#   From the HTML page register.html, the user provides information
#   that registers him or her for a software package.  This script
#   validates the input data, returns a page of error messages if
#   errors are detected, or asks for confirmation of the input data.
#
#############################################################################
#  
#   MAIN ROUTINES
#
#############################################################################

&ReadData;
if ( &ValidData ) {
	&ConfirmReg;
} else {
	&PrintErrors;
}

#############################################################################
#  
#   SUBROUTINES
#
#############################################################################

sub ReadData {
	&ReadParse;
	$name = $in{'name'};
	$email = $in{'email'};
	$platform = $in{'platform'};
	$TeXapp = $in{'TeXapp'};
	$otherApp = $in{'otherApp'};
	
	return 1;
}

# Read and parse the data passed from fill-out form:
sub ReadParse {
	local ($i, $loc, $key, $val);
	
	# Note the added else-block for command line debugging:
	if ( $ENV{'REQUEST_METHOD'} eq "GET" ) {
		$in = $ENV{'QUERY_STRING'};
	} elsif ( $ENV{'REQUEST_METHOD'} eq "POST" ) {
		for ( $i = 0; $i < $ENV{'CONTENT_LENGTH'}; $i++ ) {
			$in .= getc;
		}
	} else {
		$in = <STDIN>;
		print "$in\n";
	}
	
	# Split input string into an array of strings:
	@in = split(/&/,$in);
	
	# Process each array element individually:
	foreach $i ( 0 .. $#in ) {
		# Convert plus signs to spaces:
		$in[$i] =~ s/\+/ /g;
		# Convert %nn from hex to alphanumeric:
		$in[$i] =~ s/%(..)/pack("c",hex($1))/ge;
		# Determine the location of the equals sign:
		$loc = index($in[$i], "=");
		# Extract the key:
		$key = substr($in[$i],0,$loc);
		# Extract the value:
		$val = substr($in[$i],$loc+1);
		# Multiple values are separated by a null character:
		$in{$key} .= '\0' if ( defined($in{$key}) );
		$in{$key} .= $val;
	}
	
	return 1; 
}

# Validate the input data and accumulate the error messages:
sub ValidData {
	local ($isOkay);
	
	$isOkay = 1;
	$msgs = "";
	if ( $name eq "" ) {
		$msgs .= "error:  name omitted <P>\n";
		$isOkay = 0;
	}
	if ( $email eq "" ) {
		$msgs .= "error:  e-mail omitted <P>\n";
		$isOkay = 0;
	}
	if ( $TeXapp eq "other" && $otherApp eq "" ) {
		$msgs .= "error:  \"other\" TeX app omitted <P>\n";
		$isOkay = 0;
	}
	
	return $isOkay;
}

# No errors in the input data were found, so print an HTML page
# and request confirmation from the user:
sub ConfirmReg {
	print "Content-type: text/html\n\n";

	# Opening HTML statements:
	print "<HTML>\n";
	print "<HEAD>\n";
	print "<TITLE>Confirmation Request</TITLE>\n";
	print "</HEAD>\n";
	print "<BODY>\n\n";
	
	# Begin HTML body:
	print "<H1>Input Data</H1>\n\n";
	print "<UL>\n";
	print "  <LI>name = $name\n";
	print "  <LI>e-mail = $email\n";
	print "  <LI>platform = $platform\n";
	print "  <LI>TeX app = ";
	if ( $TeXapp eq "other" ) {
		print "$otherApp\n";
	} else {
		print "$TeXapp\n";
	}
	print "</UL>\n\n";

	# End HTML body:
	print '<FORM METHOD="POST"';
	print "\n";
	print 
'ACTION="http://osprey7.npac.syr.edu:3768/users-cgi/njm/trscavo/register.pl">';
	print "\n\n";
	print '<INPUT TYPE="hidden" NAME="name" VALUE=';
	print "\"$name\">\n";
	print '<INPUT TYPE="hidden" NAME="email" VALUE=';
	print "\"$email\">\n";
	print '<INPUT TYPE="hidden" NAME="platform" VALUE=';
	print "\"$platform\">\n";
	print '<INPUT TYPE="hidden" NAME="TeXapp" VALUE=';
	print "\"$TeXapp\">\n";
	print '<INPUT TYPE="hidden" NAME="otherApp" VALUE=';
	print "\"$otherApp\">\n";
	print "Is this correct? <P>\n";
	print '<INPUT TYPE="submit" NAME="install" VALUE="Yes">';
	print "\n";
	print '<INPUT TYPE="submit" NAME="install" VALUE="No">';
	print "\n";
	print "\n</FORM>\n";
	
	# Closing HTML statements:
	print "\n</BODY>\n";
	print "</HTML>\n";
	
	return 1;
}

# Print a page of error messages and allow the user reload the HTML form:
sub PrintErrors {
	print "Content-type: text/html\n\n";
	
	# Opening HTML statements:
	print "<HTML>\n";
	print "<HEAD>\n";
	print "<TITLE>Error Report</TITLE>\n";
	print "</HEAD>\n";
	print "<BODY>\n\n";
	
	# Begin HTML body:
	print "<H1>Input errors detected</H1>\n\n";
	print "$msgs\n";
	
	# End HTML body:
	print '<P><HR><P>';
	print "\n";
	print '<A HREF="http://newton.npac.syr.edu:3768/cps606fall96-docs/trscavo5/hw2/register.html"><EM>Try again!</EM></A>';
	print "\n";
	
	# Closing HTML statements:
	print "\n</BODY>\n";
	print "</HTML>\n";
	
	return 1;
}
