/***************************************************************************/
/* Pager.c - The Semester Programming Project of CIS 657, Operating System */
/*						  Programmer : Jungkee Kim */
/* Original developing environment : DOS 6.2/MS-C Version 7.0.		   */
/* Possible UNIX compiling : gcc -o pager pager.c			   */
/***************************************************************************/

#include <stdio.h>
#include <stdlib.h>

typedef struct	PM_type physical_memory;    /*Physical memory space*/

struct	PM_type {
	short	process;		    /*Process number*/
	short	VM;			    /*Virtual memory number*/
	};

typedef struct	Queue_type queue;	    /*Queue type of queues*/

struct	Queue_type {
	short	process;		    /*Process number*/
	queue	*next;			    /*The next link*/
	};

typedef struct	MAP_type map_table;	    /*Mapping table to */
					    /* the physical memory*/
struct	MAP_type {
	short		current;	    /*Recently used set*/
	physical_memory first;		    /*The first row set*/
	physical_memory second; 	    /*The second row set*/
	};

typedef struct	Page_table_type page_table; /*Page table*/

struct	Page_table_type {
	short	present;		    /*Present/Absent bit*/
	short	number; 		    /*Physical memory number*/
	};

/*-----------------------------Constants-----------------------------------*/
#define     MAX_PAGE	8		    /*# of maximum page*/
#define     EMPTY	-99		    /*Empty status*/
#define     ABSENT	0		    /*Absent status*/
#define     PRESENT	1		    /*Present status*/
#define     TRUE	1		    /*True status*/
#define     FALSE	0		    /*False status*/
#define     SET 	4		    /*# of set of mapping table*/
#define     FIRST	1		    /*Recent used set is first row*/
#define     SECOND	2		    /*Recent used set is second row*/
#define     START	3		    /*From empty to start status*/

void initialization();
void add_w_queue( short process );
void del_w_queue( short process );
void add_b_queue( short process );
void del_b_queue( short process );
void display_p_memory();
void switching_order( short number );
void page_fault( short number );

/*---------------------------Global variables------------------------------*/
physical_memory page[8];		    /*Physical memory table*/

queue		*wq;			    /*Waiting Queue*/
queue		*bq;			    /*Blocked Queue*/

page_table	table[16][16];		    /*Page table*/

short		run_process;		    /*Current running process */
					    /* the same as the running queue*/
short		blocked=FALSE;		    /*There is no blocked process.*/

/*long		no_of_page_fault=0;*/	    /*Number of page faults */
					    /* - not used in this version*/
map_table   mapping[4]; 		    /*Memory mapping table */
					    /* - from virtual to physical*/
main()
{
    char    buf[4];			    /*Input buffer*/
    short   process;			    /*Input process*/
    short   number;			    /*Input virtual memory number*/

    initialization();			    /*Initializing all the values*/

    while ( fscanf(stdin, "%s", buf) != EOF ) {	/*Receiving a string*/
	switch ( buf[0] ) {
	    case 'W' : {		    /*Waiting status*/
		process = (short)atoi( (char *)&buf[1] );
					/*Converting from string to integer*/
		add_w_queue(process);	/*Adding a process to waiting queue*/
		break;
	    }
	    case 'R' : {		    /*Running status*/
		process = (short)atoi( (char *)&buf[1] );
		del_w_queue(process);	    /*Deleting the waiting queue*/
		run_process = process;	    /*Setting current process */
					    /* variable 	      */
		break;
	    }
	    case 'B' : {		    /*Blocked status*/
		process = (short)atoi( (char *)&buf[1] );
		add_b_queue(process);	    /*Adding a process to blocked */
		break;			    /* queue			  */
	    }
	    case 'D' : {		    /*Displaying current physical */
		display_p_memory();	    /* memory			  */
		break;
	    }
	    default : { 		    /*processing virtual memory */
					    /* number			*/
		number = (short)atoi( (char *)&buf[0] );
		if ( table[run_process][number].present ) {
					    /*If the virtual page is pre- */
					    /* sent in physical memory,*/
		    switching_order( number );
					    /*Just switch the order.*/
		}
		else {
		    page_fault(number);     /*Processing the page fault case*/
		break;
		}
	    }
	}
    }
    /*printf("# of page fault : %d\n", no_of_page_fault);*/
				/*Printing page fault number - not used*/
}

/*-------------------------------------------------------------------------*/
/* Function initialization - Initializing the global variables. 	   */
/*-------------------------------------------------------------------------*/
void initialization()
{
    int     i, j;

    for (i=0; i<8; i++) {		    /*Physical memory table*/
	page[i].process = EMPTY;
	page[i].VM = EMPTY;
    }

    wq = (queue *)malloc( sizeof(queue) );  /*Waiting queue*/
    wq->process = EMPTY;
    wq->next = NULL;
    bq = (queue *)malloc( sizeof(queue) );  /*Blocked queue*/
    bq->process = EMPTY;
    bq->next = NULL;

    for (i=0; i<4; i++) {		    /*Mapping table*/
	mapping[i].current = EMPTY;	    /*Current status*/
	mapping[i].first.process = EMPTY;   /*The first row set*/
	mapping[i].first.VM = EMPTY;
	mapping[i].second.process = EMPTY;  /*The second row set*/
	mapping[i].second.VM = EMPTY;
    }

    for (i=0; i<16; i++)		    /*The page table*/
	for (j=0; j<16; j++) {
	    table[i][j].present = ABSENT;   /*Present/Absent bit*/
	    table[i][j].number = EMPTY;
	}
}

/*-------------------------------------------------------------------------*/
/*Function add_w_queue - Adding a process to the waiting queue. 	   */
/*-------------------------------------------------------------------------*/
void add_w_queue( short process )
{
    queue   *ptr;

    if ( wq->process == EMPTY ) {	/*If the waiting queue is empty,*/
	wq->process = process;		/*write process # to the first list.*/
	return;
    }
    if ( blocked )			/*If the process was blocked,*/
	del_b_queue( process ); 	/*delete from the blocked queue.*/

    ptr = wq;				/*Pointing to the waiting queue*/

    while ( ptr->next != NULL ) 	/*Searching the last position*/
	ptr = ptr->next;
					/*Adding to the waiting queue*/
    ptr->next = (queue *)malloc( sizeof(queue) );
    ptr = ptr->next;
    ptr->process = process;
    ptr->next = NULL;
}

/*-------------------------------------------------------------------------*/
/*Function del_w_queue - Deleting a process from the waiting queue.	   */
/*-------------------------------------------------------------------------*/
void del_w_queue( short process )
{
    queue   *ptr, *temp;

    ptr = wq;				/*Pointing to the waiting queue*/

    if ( ptr->process == process ) {	/*If the first process is matching,*/
	if ( ptr->next == NULL ) {	/*if only one node exists,*/
	    ptr->process = EMPTY;	/*just make the value to EMPTY.*/
	    return;
	}
	wq = ptr->next; 		/*Moving the waiting queue pointer */
	free(ptr);			/* to the next and free the node.*/
	return; 			/*Return to the calling function.*/
    }
					/*Searching to the last node*/
    while ( ptr->next != NULL ) {	/*Deleting a node from the queue*/
	if ( ptr->next->process == process ) {
	    temp = ptr->next;
	    ptr->next = ptr->next->next;
	    free(temp);
	    return;
	}
	ptr = ptr->next;		/*Moving to the next node*/
    }
    return;
}

/*-------------------------------------------------------------------------*/
/* Function add_b_queue - Adding a process to the blocked queue.	   */
/*-------------------------------------------------------------------------*/
void add_b_queue( short process )
{
    queue   *ptr;

    if ( bq->process == EMPTY ) {	/*If the blocked queue is empty,*/
	bq->process = process;		/*give a value to the first node */
	blocked = TRUE; 		/* and set the blocked status.*/
	return;
    }

    ptr = bq;				/*Pointing to the blocked queue*/
    while ( ptr->next != NULL ) 	/*Searching to the last queue*/
	ptr = ptr->next;
					/*Adding a node to the blocked queue*/
    ptr->next = (queue *)malloc( sizeof(queue) );
    ptr = ptr->next;
    ptr->process = process;
    ptr->next = NULL;
    return;
}

/*-------------------------------------------------------------------------*/
/* Function del_b_queue - deleting a process from the blocked queue.	   */
/*-------------------------------------------------------------------------*/
void del_b_queue( short process )
{
    queue   *ptr, *temp;

    ptr = bq;				/*Pointing to the blocked queue.*/
    if ( ptr->process == process ) {	/*If the first node is matching,*/
	if ( ptr->next == NULL ) {	/*if only one node exists,*/
	    ptr->process = EMPTY;	/*just make the value to EMPTY */
	    blocked = FALSE;		/* and reset the blocked status*/
	    return;
	}
	bq = ptr->next; 		/*Move the blocked pointer */
	free(ptr);			/* and free the node.*/
	return;
    }

    while ( ptr->next != NULL ) {	/*Searching to the last node*/
					/*If the process exists in queue,*/
	if ( ptr->next->process == process ) {
	    temp = ptr->next;		/*delete the matched process.*/
	    ptr->next = ptr->next->next;
	    free(temp);
	    return;
	}
	ptr = ptr->next;
    }
    return;
}

/*-------------------------------------------------------------------------*/
/* Function display_p_memory - Displaying the current physical memory's    */
/*			processes and virtual memory page numbers.	   */
/*-------------------------------------------------------------------------*/
void display_p_memory()
{
    short   i;

    for(i=0; i<MAX_PAGE; i++) {
	if ( page[i].process == EMPTY )     /*If the memory is empty,*/
	    printf("X ");		    /*print 'X'.*/
	else				    /*If the memory is not empty,*/
	    printf("%d,%d ", page[i].process, page[i].VM);
    }			/*print the process # and the virtual page number.*/
    printf("\n");
}

/*-------------------------------------------------------------------------*/
/* Function switching order - Switching the current refered physical memory*/
/*		    from the first row of set to the second or vice versa. */
/*-------------------------------------------------------------------------*/
void switching_order( short number )
{
    short   remainder;			/*Remainder of the virtual page #*/

    remainder = number % SET;		/*Getting a remainder value*/

    switch ( mapping[remainder].current ) { /*Checking the current status*/
	case START : {			/*The second row set is empty.*/
	    break;
	}
	case FIRST : {		/*If the last reference was the first row,*/
	    if ( mapping[remainder].first.process == run_process &&
		 mapping[remainder].first.VM == number )
		break;	    /*if the process and virtual memory's value are */
			    /* the same, just leave the loop */
			    /*or set the current reference status to SECOND.*/
	    mapping[remainder].current = SECOND;
	    break;
	}
	case SECOND : { 	/*If the last reference was the second row,*/
	    if ( mapping[remainder].second.process == run_process &&
		 mapping[remainder].second.VM == number )
		break;	    /*if the process and virtual memory's value are */
			    /* the same, just leave the loop,*/
			    /*or set the current reference status to FIRST.*/
	    mapping[remainder].current = FIRST;
	    break;
	}
	default : {	    /*Other cases are error - Not reachable*/
	    printf("Switching order error!!!\n");
	    exit(0);
	}
    }
}

/*-------------------------------------------------------------------------*/
/* Function page_fault - Allocating a process page to a new physical memory*/
/*		    space in the case of page fault.			   */
/*-------------------------------------------------------------------------*/
void page_fault( short number )
{
    short   remainder;

    /*no_of_page_fault++;*/	    /*Increasing # of page fault - Not used*/

    remainder = number % SET;		    /*Getting a remainder value*/

    switch ( mapping[remainder].current ) { /*Checking the current status*/
	case EMPTY : {	    /*If the set is empty,*/
			    /*set the current status to the START status,*/
	    mapping[remainder].current = START;
			    /*fill the first row set,*/
	    mapping[remainder].first.process = run_process;
	    mapping[remainder].first.VM = number;
			    /*set the page table's present bit,*/
	    table[run_process][number].present = PRESENT;
			    /*allocate the physical memory page,*/
	    table[run_process][number].number = remainder;
			    /*fill the physical memory table,*/
	    page[remainder].process = run_process;
	    page[remainder].VM = number;
			    /*and print the page fault message.*/
	    printf("Page Fault: (%d,%d) put in frame %d replacing (x,x)\n",
		   run_process, number, remainder);
	    break;
	}
	case START : {	    /*If the second row set is empty,*/
			    /*set the current status to the SECOND status,*/
	    mapping[remainder].current = SECOND;
			    /*fill the second row set,*/
	    mapping[remainder].second.process = run_process;
	    mapping[remainder].second.VM = number;
			    /*set the page table's present bit,*/
	    table[run_process][number].present = PRESENT;
			    /*allocate the physical memory page,*/
	    table[run_process][number].number = remainder+SET;
			    /*fill the phsical memory table,*/
	    page[remainder+SET].process = run_process;
	    page[remainder+SET].VM = number;
			    /*and print the page fault message.*/
	    printf("Page Fault: (%d,%d) put in frame %d replacing (x,x)\n",
		   run_process, number, remainder+SET);
	    break;
	}
	case FIRST : {	    /*If the last reference was the first row set,*/
			    /*allocate the current process page to the */
			    /* second row set.*/
			    /*Setting the current status to SECOND status*/
	    mapping[remainder].current = SECOND;
			    /*Resetting the old mapping table's page table*/
	    table[mapping[remainder].second.process]
		 [mapping[remainder].second.VM].present = ABSENT;
			    /*Printing the page fault message*/
	    printf("Page Fault: (%d,%d) put in frame %d replacing (%d,%d)\n",
		   run_process, number, remainder+SET,
		   mapping[remainder].second.process,
		   mapping[remainder].second.VM);
			    /*Filling the mapping table*/
	    mapping[remainder].second.process = run_process;
	    mapping[remainder].second.VM = number;
			    /*Setting the page table*/
	    table[run_process][number].present = PRESENT;
	    table[run_process][number].number = remainder+SET;
			    /*Filling the physical page table*/
	    page[remainder+SET].process = run_process;
	    page[remainder+SET].VM = number;
	    break;
	}
	case SECOND : {     /*If the last reference was the second row set,*/
			    /*allocate the current process page to the */
			    /* first row set.*/
			    /*Setting the current status to FIRST status*/
	    mapping[remainder].current = FIRST;
			    /*Resetting the old mapping table's page table*/
	    table[mapping[remainder].first.process]
		 [mapping[remainder].first.VM].present = ABSENT;
			    /*Printing the page fault message*/
	    printf("Page Fault: (%d,%d) put in frame %d replacing (%d,%d)\n",
		   run_process, number, remainder+SET,
		   mapping[remainder].first.process,
		   mapping[remainder].first.VM);
			    /*Filling the mapping table*/
	    mapping[remainder].first.process = run_process;
	    mapping[remainder].first.VM = number;
			    /*Settting the page table*/
	    table[run_process][number].present = PRESENT;
	    table[run_process][number].number = remainder;
			    /*Filling the physical page table*/
	    page[remainder].process = run_process;
	    page[remainder].VM = number;
	    break;
	}
	default : {	    /*Other cases are error - Not reachable*/
	    printf("Page faulting process error!!!\n");
	    exit(0);
	}
    }
}
