#include <stdio.h>
#include <stdlib.h>

typedef struct  PM_type physical_memory;

struct  PM_type {
	short   process;
	short   VM;
	};

typedef struct  Queue_type queue;

struct  Queue_type {
	short   process;
	queue   *next;
	};

typedef struct  FIFO_type page_list;

struct  FIFO_type {
	short       process;
	short       VM;
	page_list   *next;
	};

typedef struct  LRU_type lru_list;

struct  LRU_type {
	short       process;
	short       VM;
	lru_list    *before;
	lru_list    *next;
	};

typedef struct  MAP_type map_table;

struct  MAP_type {
	short           current;
	physical_memory first;
	physical_memory second;
	};

typedef struct  Page_table_type page_table;

struct  Page_table_type {
	short   present;
	short   number;
	};

#define     MAX_PAGE    8
#define     EMPTY       -99
#define     ABSENT      0
#define     PRESENT     1
#define     TRUE        1
#define     FALSE       0
#define     SET         4
#define     FIRST       1
#define     SECOND      2
#define     START       3

void initialization();
void add_w_queue( short process );
void del_w_queue( short process );
void add_b_queue( short process );
void del_b_queue( short process );
void display_p_memory();
void switching_order( short number );
/*void change_order( short number );*/
void page_fault( short number );

physical_memory page[8];

queue           *wq;
queue           *bq;

page_table      table[16][16];

short           run_process;

short           memory_full=FALSE;

short           blocked=FALSE;

long            no_of_page_fault=0;

lru_list    *lru;
lru_list    *last;

map_table   mapping[4];

main()
{
    char    buf[4];
    short   process;
    short   number;
    FILE    *fp;        /*xxxxx*/

    initialization();

    fp = fopen("myinput", "rt");

    while ( (fscanf(fp, "%s", buf)) != EOF /*xxx move*/) {
	switch ( buf[0] ) {
	    case 'W' : {
		process = (short)atoi( (char *)&buf[1] );
		add_w_queue(process);
		break;
	    }
	    case 'R' : {
		process = (short)atoi( (char *)&buf[1] );
		del_w_queue(process);
		run_process = process;
		break;
	    }
	    case 'B' : {
		process = (short)atoi( (char *)&buf[1] );
		add_b_queue(process);
		break;
	    }
	    case 'D' : {
		display_p_memory();
		break;
	    }
	    default : {
		number = (short)atoi( (char *)&buf[0] );
		if ( table[run_process][number].present ) {
		    /*change_order( number );*/
		    switching_order( number );
		}
		else {
		    page_fault(number);
		break;
		}
	    }
	}
    }
    fclose( fp );
    printf("# of page fault : %d\n", no_of_page_fault);
}

void initialization()
{
    int     i, j;

    for (i=0; i<8; i++) {
	page[i].process = EMPTY;
	page[i].VM = EMPTY;
    }

    wq = (queue *)malloc( sizeof(queue) );
    wq->process = EMPTY;
    wq->next = NULL;
    bq = (queue *)malloc( sizeof(queue) );
    bq->process = EMPTY;
    bq->next = NULL;
    /*lru = (lru_list *)malloc( sizeof(lru_list) );
    lru->process = EMPTY;
    lru->before = NULL;
    lru->next = NULL;
    */
    for (i=0; i<4; i++) {
	mapping[i].current = EMPTY;
	mapping[i].first.process = EMPTY;
	mapping[i].first.VM = EMPTY;
	mapping[i].second.process = EMPTY;
	mapping[i].second.VM = EMPTY;
    }

    for (i=0; i<16; i++)
	for (j=0; j<16; j++) {
	    table[i][j].present = ABSENT;
	    table[i][j].number = EMPTY;
	}
}

void add_w_queue( short process )
{
    queue   *ptr;

    /*printf("W-%d\n", process);*/
    if ( wq->process == EMPTY ) {
	wq->process = process;
	return;
    }
    if ( blocked )
	del_b_queue( process );
    ptr = wq;
    while ( ptr->next != NULL )
	ptr = ptr->next;
    ptr->next = (queue *)malloc( sizeof(queue) );
    ptr = ptr->next;
    ptr->process = process;
    ptr->next = NULL;
}

void del_w_queue( short process )
{
    queue   *ptr, *temp;

    /*printf("w-(%d)\n", process);*/
    ptr = wq;
    if ( ptr->process == process ) {
	if ( ptr->next == NULL ) {
	    ptr->process = EMPTY;
	    return;
	}
	wq = ptr->next;
	free(ptr);
	return;
    }
    while ( ptr->next != NULL ) {
	if ( ptr->next->process == process ) {
	    temp = ptr->next;
	    ptr->next = ptr->next->next;
	    free(temp);
	    return;
	}
	ptr = ptr->next;
    }
    return;
}

void add_b_queue( short process )
{
    queue   *ptr;

    if ( bq->process == EMPTY ) {
	bq->process = process;
	blocked = TRUE;
	return;
    }
    ptr = bq;
    while ( ptr->next != NULL )
	ptr = ptr->next;
    ptr->next = (queue *)malloc( sizeof(queue) );
    ptr = ptr->next;
    ptr->process = process;
    ptr->next = NULL;
    return;
    /*printf("B-%d\n", process);*/
}

void del_b_queue( short process )
{
    queue   *ptr, *temp;

    /*printf("w-(%d)\n", process);*/
    ptr = bq;
    if ( ptr->process == process ) {
	if ( ptr->next == NULL ) {
	    ptr->process = EMPTY;
	    blocked = FALSE;
	    return;
	}
	bq = ptr->next;
	free(ptr);
	return;
    }
    while ( ptr->next != NULL ) {
	if ( ptr->next->process == process ) {
	    temp = ptr->next;
	    ptr->next = ptr->next->next;
	    free(temp);
	    return;
	}
	ptr = ptr->next;
    }
    return;
}

void display_p_memory()
{
    short   i;

    for(i=0; i<8; i++) {
	if ( page[i].process == EMPTY )
	    printf("X ");
	else
	    printf("%d,%d ", page[i].process, page[i].VM);
    }
    printf("\n");
}

void switching_order( short number )
{
    short   remainder;

    remainder = number % SET;
    switch ( mapping[remainder].current ) {
	case START : {
	    break;
	}
	case FIRST : {
	    if ( mapping[remainder].first.process == run_process &&
		 mapping[remainder].first.VM == number )
		break;
	    mapping[remainder].current = SECOND;
	    break;
	}
	case SECOND : {
	    if ( mapping[remainder].second.process == run_process &&
		 mapping[remainder].second.VM == number )
		break;
	    mapping[remainder].current = FIRST;
	    break;
	}
	default : {
	    printf("Switching order error!!!\n");
	    exit(0);
	}
    }
}


/*
void change_order( short number )
{
    lru_list    *ptr;

    ptr = lru;
    if ( ptr->process == run_process && ptr->VM == number )
	return;
    while ( ptr->process != run_process || ptr->VM != number)
	ptr = ptr->next;
    if ( ptr->next == NULL )
	last = last->before;
    ptr->before->next = ptr->next;
    ptr->next->before = ptr->before;
    ptr->next = lru;
    lru->before = ptr;
    lru = ptr;
}
*/

void page_fault( short number )
{
    short   remainder;

    no_of_page_fault++;

    remainder = number % SET;
    switch ( mapping[remainder].current ) {
	case EMPTY : {
	    mapping[remainder].current = START;
	    mapping[remainder].first.process = run_process;
	    mapping[remainder].first.VM = number;
	    table[run_process][number].present = PRESENT;
	    table[run_process][number].number = remainder;
	    page[remainder].process = run_process;
	    page[remainder].VM = number;
	    printf("Page Fault: (%d,%d) put in frame %d replacing (x,x)\n",
		   run_process, number, remainder);
	    break;
	}
	case START : {
	    mapping[remainder].current = SECOND;
	    mapping[remainder].second.process = run_process;
	    mapping[remainder].second.VM = number;
	    table[run_process][number].present = PRESENT;
	    table[run_process][number].number = remainder+SET;
	    page[remainder+SET].process = run_process;
	    page[remainder+SET].VM = number;
	    printf("Page Fault: (%d,%d) put in frame %d replacing (x,x)\n",
		   run_process, number, remainder+SET);
	    break;
	}
	case FIRST : {
	    mapping[remainder].current = SECOND;
	    table[mapping[remainder].second.process]
		 [mapping[remainder].second.VM].present = ABSENT;
	    printf("Page Fault: (%d,%d) put in frame %d replacing (%d,%d)\n",
		   run_process, number, remainder+SET,
		   mapping[remainder].second.process,
		   mapping[remainder].second.VM);
	    mapping[remainder].second.process = run_process;
	    mapping[remainder].second.VM = number;
	    table[run_process][number].present = PRESENT;
	    table[run_process][number].number = remainder+SET;
	    page[remainder+SET].process = run_process;
	    page[remainder+SET].VM = number;
	    break;
	}
	case SECOND : {
	    mapping[remainder].current = FIRST;
	    table[mapping[remainder].first.process]
		 [mapping[remainder].first.VM].present = ABSENT;
	    printf("Page Fault: (%d,%d) put in frame %d replacing (%d,%d)\n",
		   run_process, number, remainder+SET,
		   mapping[remainder].first.process,
		   mapping[remainder].first.VM);
	    mapping[remainder].first.process = run_process;
	    mapping[remainder].first.VM = number;
	    table[run_process][number].present = PRESENT;
	    table[run_process][number].number = remainder;
	    page[remainder].process = run_process;
	    page[remainder].VM = number;
	    break;
	}
	default : {
	    printf("Page faulting process error!!!\n");
	    exit(0);
	}
    }
}

/*
void page_fault( short number )
{
    lru_list    *ptr;
    short       count=0;

    no_of_page_fault++;

    if (memory_full) {
	ptr = last;
	table[ptr->process][ptr->VM].present = ABSENT;
	table[run_process][number].present = PRESENT;
	table[run_process][number].number =
	    table[ptr->process][ptr->VM].number;
	page[table[run_process][number].number].process = run_process;
	page[table[run_process][number].number].VM = number;
	printf("Page Fault: (%d,%d) put in frame %d replacing (%d,%d)\n",
		run_process, number, table[run_process][number].number,
		ptr->process, ptr->VM);

	ptr->process = run_process;
	ptr->VM = number;
	change_order( number );
	return;
    }
    else {
	if (lru->process == EMPTY) {
	    lru->process = run_process;
	    lru->VM = number;
	}
	else {
	    ptr = (lru_list *)malloc( sizeof(lru_list) );
	    ptr->before = NULL;
	    ptr->next = lru;
	    lru->before = ptr;
	    lru = ptr;
	    ptr->process = run_process;
	    ptr->VM = number;
	    while ( ptr->next != NULL ) {
		count++;
		ptr = ptr->next;
	    }
	    if ( count == MAX_PAGE - 1 ) {
		last = ptr;
		memory_full = TRUE;
	    }
	}
	table[run_process][number].present = PRESENT;
	table[run_process][number].number = count;
	page[count].process = run_process;
	page[count].VM = number;
	printf("Page Fault: (%d,%d) put in frame %d replacing (x,x)\n",
		run_process, number, count);
    }
    return;
}
*/
