import java.applet.*;
import java.awt.*; 
import java.lang.Math; 
import java.util.*;

public class GraphThree extends java.applet.Applet { 
    GraphDraw           graph1;
    GraphDraw           graph2;
    GraphDraw           graph3;

    public void init() {
        setLayout(new GridLayout(3, 1, 10, 5));
	graph1 = new GraphDraw(this);
        graph2 = new GraphDraw(this);
	graph3 = new GraphDraw(this);
	add(graph1);
	graph1.setSine();
	add(graph2);
	graph2.setRectangular();
	add(graph3);
	graph3.setTriangle();
    }
}

class GraphDraw extends Panel {
    GraphThree            parent;
    GraphDrawCanvas	paper; 
    GraphDrawPanel	board;
 
    GraphDraw(GraphThree parent) {
        this.parent = parent;
	setLayout(new BorderLayout()); 
	paper = new GraphDrawCanvas(this); 
	board = new GraphDrawPanel(this); 
	add("Center", paper); 
	add("East", board);
	paper.setBackground(Color.black); 
        paper.setForeground(Color.green); 
	board.setBackground(Color.lightGray); 
    } 
 
    public void setSine() {
        paper.setSine();
    }

    public void setRectangular() {
        paper.setRectangular();
    }

    public void setTriangle() {
        paper.setTriangle();
    }

    public void paint (Graphics g) { 
	paper.paint(g); 
	board.paint(g); 
    } 
 
    public void start() { 
	paper.start(); 
	board.start(); 
    } 
     
    public void stop() { 
	paper.stop(); 
    } 
 
    public void update(Graphics g) { 
	paper.paint(g); 
    } 
} 

class GraphDrawCanvas extends Canvas implements Runnable { 
 
    GraphDraw parent;
    int shape;
    Thread thread;				//Thread variable 
    double dx = 0.0;
    double dy = 1.0;
    double df = 1.0;

    public GraphDrawCanvas(GraphDraw parent) {
	this.parent = parent;
	thread = new Thread(this);
    } 
 
    public void setSine() {
        shape = 1;
    }

    public void setRectangular() {
        shape = 2;
    }

    public void setTriangle() {
        shape = 3;
    }

    public void start() { 
 
	if (thread.isAlive()) {		//Checking the life of thread 
	    thread.resume();		//If other page reference, 
	}				// resume the thread. 
 
	else {				//Start thread from begining 
	    thread.start(); 
	} 
    } 
 
    public void stop() {		//Stop thread. 
	thread.suspend(); 
    } 
 
    public void destroy() {		//Destroy thread. 
	thread.stop(); 
    } 
 
    public void run() { 
        repaint(); 
    } 
 
    double f(double x) { 
        switch (shape) {
	    case 1 :
                return (Math.sin(x*df/10 +dx) * dy + 2 )* size().height / 4;
	    case 2:
                double y = x * df / 10 + dx;
	        if (Math.floor(y / Math.PI) % 2 == 0) {
		    return (1 * dy + 2) * size().height / 4;
		}
		else {
		    return (-1 * dy + 2) * size().height / 4;
		}
	    case 3:
	        y = x * df / 10 + dx;
	        double length = Math.PI / 2 * 10 / df;
		double height = dy * size().height / 4;
		double slope = height / length;
		double period = 4 * length;
		double d = dx * 2 / Math.PI * length;
		if ( d < 0 ) { x += period; }
		if (Math.floor(y / Math.PI * 2) % 4 == 0) {
		    return slope * ( (x+d) % period ) + size().height / 2;
		}
		else if (Math.floor(y / Math.PI * 2) % 4 == 3) {
		    return slope * (( (x+d) - 3 * length) % period) + size().height / 2 - height;
		}
		else {
		  return -1 * slope * (( x+d - length ) % period) + size().height / 2 + height;
		}
	    default:
                return 0;
	}
    }
 
    public void paint(Graphics g) { 
        for (int x = 0 ; x < size().width ; x++) { 
            g.drawLine(x, (int)f(x), x + 1, (int)f(x + 1)); 
        } 
    } 
 
    public void update_phase(int flag) { 
        switch( flag ) { 
	    case 1 : dx += Math.PI / 4; 
                     break; 
	    case 2 : dx -= Math.PI / 4;; 
                     break; 
	    default : break; 
	}
	if ( dx < 0 ) { dx += Math.PI * 2; } 
        repaint(); 
    }
  
    public void update_amplitude(int flag) {
      switch( flag ) {
          case 1 : dy -= 0.1;
	           break;
          case 2 : dy += 0.1;
                   break;
          default : break;
      }
      if ( dy < 0 ) {
          dy = 0;
      }
      if ( dy > 2 ) {
	  dy = 2;
      }
      repaint();
    }

    public void update_frequency(int flag) {
      switch( flag ) {
          case 1 : df -= 0.1;
	           break;
          case 2 : df += 0.1;
                   break;
          default : break;
      }
      if ( df < 0 ) {
	  df = 0;
      }
      repaint();
    }

} 
 
class GraphDrawPanel extends Panel { 
    GraphDraw parent; 
 
    GraphDrawPanel(GraphDraw parent) { 
	this.parent = parent; 
	setLayout(new GridLayout(3, 2, 5, 5)); 
 
	add(new Button("<"));
	add(new Button(">"));
        add(new Button("-"));
        add(new Button("+"));
	add(new Button("L"));
	add(new Button("H"));
    } 
 
    public void start() { 
    } 
 
    public void paint(Graphics g) { 
    } 
 
    public boolean action(Event evt, Object arg) { 
        if (evt.target instanceof Button) 
            changePhase((String)arg); 
        return true; 
    } 
 
    void changePhase(String bname) { 
        if (bname.equals("<")) parent.paper.update_phase(1); 
        else if (bname.equals(">")) parent.paper.update_phase(2);
	else if (bname.equals("-")) parent.paper.update_amplitude(1);
	else if (bname.equals("+")) parent.paper.update_amplitude(2);
	else if (bname.equals("L")) parent.paper.update_frequency(1);
	else if (bname.equals("H")) parent.paper.update_frequency(2);
    } 
}
