//------------------------------------------------------------------------
// GraphWaves Applet
// This applet draws two kinds of sound waves and makes it possible to
// adjust graph's phase, amplitude and frequency. If user clicks on the
// 'Combine' button, it combines two graphs and draws a new graph. The
// new graph reflects the mean value of the two graphs on the each point.
// It does not mean adding two different sounds. It is used for new patern.
// If the user clicks 'Play' button, it will generate 1 second length
// 8 bit 8000Hz sampling mu-law sound.
//
// INPUT: HTML parameters of graph kind and user mouse clicking
//
// OUTPUT: graphs and 1 second length 8-bit mu-law audio sound.
//
// @author Jungkee Kim
// @version 1.0D, May 1997

import java.applet.*;
import java.awt.*; 
import java.lang.Math; 
import java.util.*;

public class GraphWaves extends java.applet.Applet {
  GraphDraw           graph1, graph2;             // sound graph 1 and 2
  GraphSum            soundgraph;                 // combined graph
  String              wave1, wave2;               // kinds of soudn waves

  public void init() {
    wave1 = getParameter("wave1");                // get parameter from HTML
    wave2 = getParameter("wave2");
    if (wave1 == null)                            // if no parameter passed
      wave1 = "Rectangular";
    if (wave2 == null)
      wave2 = "Triangle";
    setLayout(new GridLayout(3, 1, 2, 2));
    graph1 = new GraphDraw(this);                 // graph 1
    graph2 = new GraphDraw(this);                 // graph 2
    soundgraph = new GraphSum(this);              // combined graph
    add(graph1);
    if (wave1.equals("Sine")) graph1.setSine();
    else if (wave1.equals("Rectangular")) graph1.setRectangular();
    else if (wave1.equals("Triangle")) graph1.setTriangle();
    add(graph2);
    if (wave2.equals("Sine")) graph2.setSine();
    else if (wave2.equals("Rectangular")) graph2.setRectangular();
    else if (wave2.equals("Triangle")) graph2.setTriangle();
    add(soundgraph);
  }
}

/* This class draws the combined sound graph */
class GraphSum extends Panel {
  GraphWaves          parent;
  GraphSumCanvas      paper;                       // graph canvas
  PlayControlPanel    control;                     // control panel
  public static int   status = 0;                  // Combined button status
  
  GraphSum(GraphWaves parent) {
    this.parent = parent;
    setLayout(new BorderLayout());
    paper = new GraphSumCanvas(this); 
    control = new PlayControlPanel(this);
    add("Center", paper); 
    add("South", control);
    paper.setBackground(Color.black); 
    paper.setForeground(Color.red); 
    control.setBackground(new Color(0xFFEBCD));
  }

  void setSum() {
    status = 1;                                     // Combined button clicked
  }
 
  public void paint (Graphics g) { 
    paper.paint(g);
  } 
  
  public void start() { 
    paper.start(); 
  } 
  
  public void stop() { 
    paper.stop(); 
  } 
  
  public void update(Graphics g) { 
    paper.paint(g); 
  } 
}

/* This class draws a graph */
class GraphSumCanvas extends Canvas implements Runnable { 
  
  GraphSum parent;
  Thread thread;				//Thread variable 
  
  public GraphSumCanvas(GraphSum parent) {
    this.parent = parent;
    thread = new Thread(this);
  } 
  
  public void start() { 
    
    if (thread.isAlive()) {		//Checking the life of thread 
      thread.resume();		        //If other page reference, 
    }				        // resume the thread. 
    
    else {				//Start thread from begining 
      thread.start(); 
    } 
  } 
  
  public void stop() {		        //Stop thread. 
    thread.suspend(); 
  } 
  
  public void destroy() {		//Destroy thread. 
    thread.stop(); 
  } 
  
  public void run() { 
    repaint(); 
  } 
  
  /* This method gets the mean value graph of the two graphs */
  double f(double x) {
    switch(parent.status) {
    case 0:
      return size().height / 2;                    //initial graph is a line
    case 1:
      return (parent.parent.graph1.paper.f(x) +
	      parent.parent.graph2.paper.f(x)) / 2;
    default:
      return 0;
    }
  }
  
  public void paint(Graphics g) { 
    for (int x = 0 ; x < size().width ; x++) { 
      g.drawLine(x, (int)f(x), x + 1, (int)f(x + 1)); 
    }
    if (parent.status == 0) {
      g.drawString("None", 5, 10);
    }
    else if (parent.status == 1) {
      g.drawString("Combined", 5, 10);
    }
    else { g.drawString("Hey! etc...", 5, 10); }
  } 
}

/* This class shows two control buttons, 'Combine' and 'Play' */
class PlayControlPanel extends Panel {
  GraphSum parent;

  PlayControlPanel(GraphSum parent) {
    this.parent = parent;
    setLayout(new GridLayout(1, 2, 2, 2));
    add(new Button("Combine"));
    add(new Button("Play"));
  }

  public boolean action(Event evt, Object arg) {
    if (evt.target instanceof Button) {
      getWave((String)arg);
      return true;
    }
    else return false;
  }

  void getWave(String cname) {
    if (cname.equals("Combine")) {
      parent.parent.soundgraph.setSum();
    }
    else if (cname.equals("Play")) {
      PlaySound();
    }
    else {}
    parent.parent.soundgraph.paper.repaint();
  }

  void PlaySound() {
    SecondSound stream = new SecondSound(this);
    stream.run();
  }
}
