import java.applet.*;
import java.awt.*; 
import java.lang.Math; 
import java.util.*;

public class SoundGenerator extends java.applet.Applet {
  GraphWaves          graphs;
  GraphControlPanel   control;

  public void init() {
    setLayout(new BorderLayout(5, 5));
    graphs = new GraphWaves(this);
    control = new GraphControlPanel(this);
    add("Center", graphs);
    add("South", control);
    control.setBackground(Color.white);
  }
}

class GraphWaves extends Panel {
  SoundGenerator      parent;
  GraphDraw           graphSine;
  GraphDraw           graphRectangular;
  GraphDraw           graphTriangle;
  GraphSum            graphSound;

  public GraphWaves(SoundGenerator parent) {
    this.parent = parent;
    setLayout(new GridLayout(4, 1, 10, 5));
    graphSine = new GraphDraw(this);
    graphRectangular = new GraphDraw(this);
    graphTriangle = new GraphDraw(this);
    graphSound = new GraphSum(this);
    add(graphSine);
    graphSine.setSine();
    add(graphRectangular);
    graphRectangular.setRectangular();
    add(graphTriangle);
    graphTriangle.setTriangle();
    add(graphSound);
  }
}

class GraphDraw extends Panel {
  GraphWaves          parent;
  GraphDrawCanvas     paper; 
  GraphDrawPanel      board;
  
  GraphDraw(GraphWaves parent) {
    this.parent = parent;
    setLayout(new BorderLayout(5, 5));
    paper = new GraphDrawCanvas(this); 
    board = new GraphDrawPanel(this); 
    add("Center", paper); 
    add("East", board);
    paper.setBackground(Color.black); 
    paper.setForeground(Color.green); 
    board.setBackground(Color.orange);
    //board.setForeground(Color.magenta);
  } 
  
  public void setSine() {
    paper.setSine();
  }
  
  public void setRectangular() {
    paper.setRectangular();
  }
  
  public void setTriangle() {
    paper.setTriangle();
  }
  
  public void paint (Graphics g) { 
    paper.paint(g); 
    board.paint(g); 
  } 
  
  public void start() { 
    paper.start(); 
    board.start(); 
  } 
  
  public void stop() { 
    paper.stop(); 
  } 
  
  public void update(Graphics g) { 
    paper.paint(g); 
  } 
}

class kindGraphs {
  public static final int none = 0;
  public static final int sine = 1;
  public static final int rectangular = 2;
  public static final int triangle = 4;
  public static final int sr = 3;
  public static final int rt = 6;
  public static final int ts = 5;
  public static final int srt = 7;
}

class GraphSum extends Panel {
  GraphWaves          parent;
  GraphSumCanvas      paper;
  public static int   status;
  
  GraphSum(GraphWaves parent) {
    this.parent = parent;
    setLayout(new BorderLayout());
    paper = new GraphSumCanvas(this); 
    add("Center", paper); 
    paper.setBackground(Color.black); 
    paper.setForeground(Color.red); 
  }

  void setSum() {
    status = kindGraphs.none;
  }

  void setSum(int s) {
    status = s;
  }

  public void paint (Graphics g) { 
    paper.paint(g);
  } 
  
  public void start() { 
    paper.start(); 
  } 
  
  public void stop() { 
    paper.stop(); 
  } 
  
  public void update(Graphics g) { 
    paper.paint(g); 
  } 
}

class GraphDrawCanvas extends Canvas implements Runnable { 
  
  GraphDraw parent;
  int shape;
  Thread thread;				//Thread variable 
  double dx = 0.0;
  double dy = 1.0;
  double df = 1.0;
  
  public GraphDrawCanvas(GraphDraw parent) {
    this.parent = parent;
    thread = new Thread(this);
  } 
  
  public void setSine() {
    shape = 1;
  }
  
  public void setRectangular() {
    shape = 2;
  }
  
  public void setTriangle() {
    shape = 3;
  }
  
  public void start() { 
    
    if (thread.isAlive()) {		//Checking the life of thread 
      thread.resume();		        //If other page reference, 
    }				        // resume the thread. 
    
    else {				//Start thread from begining 
      thread.start(); 
    } 
  } 
  
  public void stop() {                  //Stop thread. 
    thread.suspend(); 
  } 
  
  public void destroy() {		//Destroy thread. 
    thread.stop(); 
  } 
  
  public void run() { 
    repaint(); 
  } 
  
  double f(double x) { 
    switch (shape) {
    case 1 :
      return (Math.sin(x*df/10 +dx) * dy + 2 )* size().height / 4;
    case 2:
      double y = x * df / 10 + dx;
      if (Math.floor(y / Math.PI) % 2 == 0) {
	return (1 * dy + 2) * size().height / 4;
      }
      else {
	return (-1 * dy + 2) * size().height / 4;
      }
    case 3:
      y = x * df / 10 + dx;
      double length = Math.PI / 2 * 10 / df;
      double height = dy * size().height / 4;
      double slope = height / length;
      double period = 4 * length;
      double d = dx * 2 / Math.PI * length;
      if ( d < 0 ) { x += period; }
      if (Math.floor(y / Math.PI * 2) % 4 == 0) {
	return slope * ( (x+d) % period ) + size().height / 2;
      }
      else if (Math.floor(y / Math.PI * 2) % 4 == 3) {
	return slope * (( (x+d) - 3 * length) % period) + size().height / 2 - height;
      }
      else {
	return -1 * slope * (( x+d - length ) % period) + size().height / 2 + height;
      }
    default:
      return 0;
    }
  }
  
  public void paint(Graphics g) { 
    for (int x = 0 ; x < size().width ; x++) { 
      g.drawLine(x, (int)f(x), x + 1, (int)f(x + 1)); 
    } 
  } 
  
  public void update_phase(int flag) { 
    switch( flag ) { 
    case 1 : dx += Math.PI / 4; 
      break; 
    case 2 : dx -= Math.PI / 4;; 
      break; 
    default : break; 
    }
    if ( dx < 0 ) { dx += Math.PI * 2; } 
    repaint(); 
  }
  
  public void update_amplitude(int flag) {
    switch( flag ) {
    case 1 : dy -= 0.1;
      break;
    case 2 : dy += 0.1;
      break;
    default : break;
    }
    if ( dy < 0 ) {
      dy = 0;
    }
    if ( dy > 2 ) {
      dy = 2;
    }
    repaint();
  }
  
  public void update_frequency(int flag) {
    switch( flag ) {
    case 1 : df -= 0.1;
      break;
    case 2 : df += 0.1;
      break;
    default : break;
    }
    if ( df < 0 ) {
      df = 0;
    }
    repaint();
  }
  
} 

class GraphDrawPanel extends Panel { 
  GraphDraw parent; 
  
  GraphDrawPanel(GraphDraw parent) { 
    this.parent = parent; 
    setLayout(new GridLayout(3, 2, 5, 5)); 
    
    add(new Button("<"));
    add(new Button(">"));
    add(new Button("-"));
    add(new Button("+"));
    add(new Button("L"));
    add(new Button("H"));
  } 
  
  public void start() { 
  } 
  
  public void paint(Graphics g) { 
  } 
  
  public boolean action(Event evt, Object arg) { 
    if (evt.target instanceof Button) 
      changePhase((String)arg); 
    return true; 
  } 
  
  void changePhase(String bname) { 
    if (bname.equals("<")) parent.paper.update_phase(1); 
    else if (bname.equals(">")) parent.paper.update_phase(2);
    else if (bname.equals("-")) parent.paper.update_amplitude(1);
    else if (bname.equals("+")) parent.paper.update_amplitude(2);
    else if (bname.equals("L")) parent.paper.update_frequency(1);
    else if (bname.equals("H")) parent.paper.update_frequency(2);
  } 
}

class GraphSumCanvas extends Canvas implements Runnable { 
  
  GraphSum parent;
  Thread thread;				//Thread variable 
  
  public GraphSumCanvas(GraphSum parent) {
    this.parent = parent;
    thread = new Thread(this);
  } 
  
  public void start() { 
    
    if (thread.isAlive()) {		//Checking the life of thread 
      thread.resume();		        //If other page reference, 
    }				        // resume the thread. 
    
    else {				//Start thread from begining 
      thread.start(); 
    } 
  } 
  
  public void stop() {		        //Stop thread. 
    thread.suspend(); 
  } 
  
  public void destroy() {		//Destroy thread. 
    thread.stop(); 
  } 
  
  public void run() { 
    repaint(); 
  } 
  
  double f(double x) { 
    switch(parent.status) {
    case kindGraphs.none:
      return size().height / 2;
    case kindGraphs.sine:
      return parent.parent.graphSine.paper.f(x);
    case kindGraphs.rectangular:
      return parent.parent.graphRectangular.paper.f(x);
    case kindGraphs.triangle:
      return parent.parent.graphTriangle.paper.f(x);
    case kindGraphs.sr:
      return (parent.parent.graphSine.paper.f(x) + 
	      parent.parent.graphRectangular.paper.f(x)) / 2;
    case kindGraphs.rt:
      return (parent.parent.graphRectangular.paper.f(x) +
	      parent.parent.graphTriangle.paper.f(x)) / 2;
    case kindGraphs.ts:
      return (parent.parent.graphTriangle.paper.f(x) +
	      parent.parent.graphSine.paper.f(x)) / 2;
    case kindGraphs.srt:
      return (parent.parent.graphSine.paper.f(x) +
	      parent.parent.graphRectangular.paper.f(x) +
	      parent.parent.graphTriangle.paper.f(x)) / 3;
    default:
      return 0; //size().height / 2;
    }
  }
  
  public void paint(Graphics g) { 
    for (int x = 0 ; x < size().width ; x++) { 
      g.drawLine(x, (int)f(x), x + 1, (int)f(x + 1)); 
    }
    if (parent.status == kindGraphs.none) {
      g.drawString("None", 5, 10);
    }
    else if (parent.status == kindGraphs.sine) {
      g.drawString("Sine", 5, 10);
    }
    else if (parent.status == kindGraphs.rectangular) {
      g.drawString("Rectangular", 5, 10);
    }
    else if (parent.status == kindGraphs.triangle) {
      g.drawString("Triangle", 5, 10);
    }
    else if (parent.status == kindGraphs.sr) {
      g.drawString("S+R", 5, 10);
    }
    else if (parent.status == kindGraphs.rt) {
      g.drawString("R+T", 5, 10);
    }
    else if (parent.status == kindGraphs.ts) {
      g.drawString("S+T", 5, 10);
    }
    else if (parent.status == kindGraphs.srt) {
      g.drawString("S+R+T", 5, 10);
    }
    else { g.drawString("Hey! etc...", 5, 10); }
  } 
}

class GraphControlPanel extends Panel {
  SoundGenerator parent;

  GraphControlPanel(SoundGenerator parent) {
    this.parent = parent;
    setLayout(new FlowLayout());
    add(new Button("S"));
    add(new Button("R"));
    add(new Button("T"));
    add(new Button("SR"));
    add(new Button("RT"));
    add(new Button("TS"));
    add(new Button("SRT"));
    add(new Button("Play"));
  }

  public boolean action(Event evt, Object arg) {
    if (evt.target instanceof Button) {
      getWave((String)arg);
      return true;
    }
    else return false;
  }

  void getWave(String cname) {
    if (cname.equals("S")) {
      parent.graphs.graphSound.setSum(kindGraphs.sine);
    }
    else if (cname.equals("R")) {
      parent.graphs.graphSound.setSum(kindGraphs.rectangular);
    }
    else if (cname.equals("T")) {
      parent.graphs.graphSound.setSum(kindGraphs.triangle);
    }
    else if (cname.equals("SR")) {
      parent.graphs.graphSound.setSum(kindGraphs.sr);
    }
    else if (cname.equals("RT")) {
      parent.graphs.graphSound.setSum(kindGraphs.rt);
    }
    else if (cname.equals("TS")) {
      parent.graphs.graphSound.setSum(kindGraphs.ts);
    }
    else if (cname.equals("SRT")) {
      parent.graphs.graphSound.setSum(kindGraphs.srt);
    }
    else if (cname.equals("Play")) {
      PlaySound();
    }
    else {}
    parent.graphs.graphSound.paper.repaint();
  }

  void PlaySound() {
    SecondSound stream = new SecondSound(this);
    stream.run();
  }
}
