//=========================================================================
//
// class			URLAudioProvider
//
// Author:			Grzegorz Lewandowski
// Created:			09/23/97
//
//-------------------------------------------------------------------------

import java.io.*;
import java.net.*;

class URLAudioProvider extends AudioProvider
{
	InputStream inputStream;
	URL url;

	URLAudioProvider( AudioDecoder dec, AudioIOStream str, String urlSpec )
	{
		super( dec, str );
		try
		{
			url = new URL( urlSpec );
		}
		catch( MalformedURLException e )
		{
			System.err.println( urlSpec + " is not a valid URL" );
			System.exit( -1 );
		}

		try
		{
			inputStream = url.openStream();
		}
		catch( IOException e )
		{
			System.err.println( "ERROR: " + e.getMessage() );
			System.exit( -1 );
		}

		// this should be in ADPCMdecoder
		// will be moved there later
		byte[] Header;
		Header = new byte[ 26 ];
		try
		{
			int nRate;
			byte nChannels, nBits;
			inputStream.read( Header );
			nRate = Header[ 12 ] * 256 + Header[ 13 ];
			nChannels = Header[ 14 ];
			nBits = Header[ 15 ];
			System.out.println( "Rate: " + nRate );
			System.out.println( "Channels: " + nChannels );
			System.out.println( "Bits per sample: " + nBits );
		}
		catch( IOException e )
		{
			System.err.println( "URLAudioProvider: Caught IOException: " + e.getMessage() );
			System.exit( -1 );
		}
	}

	public void close()
	{
		try
		{
			inputStream.close();
		}
		catch( IOException e )
		{
			System.err.println( "URLAudioProvider: IOException " + e );
		}
	}

	protected boolean putAudio()
	{
		int nResult = 0;
		int nLength = inBuffer.length;
		int nOffset = 0;
		boolean bReturn;

		
		try
		{
			while( ( nLength > 0 ) && ( nResult != -1 ) )
			{
				nResult = inputStream.read( inBuffer, nOffset, nLength );
				nOffset += nResult;
				nLength -= nResult;
			}
		}
		catch( IOException e )
		{
			System.err.println( "URLAudioProvider: Caught IOException: " + e.getMessage() );
			bReturn = false;
		}

		if( nResult != -1 )
		{
			bReturn = true;
			// System.out.println( "URLAudioProvider: " + nResult + " bytes read" );
		}
		else
		{
			bReturn = false;
			// System.out.println( "URLAudioProvider: EOF" );
		}
		return bReturn;
	}
}