

typedef struct malloc_hdr {
	struct malloc_hdr *next;
	unsigned int size;
} malloc_hdr;

approximate definition of malloc:

	malloc_hdr *p; 
      <This is a pointer to the start of the
       free list of available blocks.>

	Search the free list until a large enough
	block is found.

	malloc_hdr *preceding_block;
		<This points to the block just
		preceding the one that is wanted.

	preceding block -> next = p -> next;
	return ++p;

end of approximate definition of malloc.

for example, 

			struct fred {...};
			fred *p = malloc(sizeof(fred)); 

NOW HERE ARE THE FILES FOR THE SAFER VERSIONS 
OF MALLOC AND FREE.

new.h

typedef long align;
#define MAGIC ((align) ) 0xabcd1234L)

typedef struct wrapper {
	align signature;
	int line_number;
	char * file_name;
} wrapper;

and here is

new.c

#include <stdio.h>
#include <stdlib.h>
#include "new.h"

#undef malloc
#undef free

static FILE *Logfile;

void *new(size_t size, char *file, int line) 
}
	wrapper *wp;

	if (!Logfile && 
		!(Logfile = fopen("new.log", "w"))) {
		fprintf(stderr, 
              "new: Can't open log file!\n");
		exit(1);
	}
	fprintf(Logfile, "%s: %d ", file line):

	if (wp = (wrapper *)
		malloc(sizeof(wrapper) + size)) {
	
		wp -> signature = MAGIC;
		wp -> line_number = line;
		wp -> file_name = file;

		fprintf(Logfile, 
           "(Allocating memory at %p)\n", wp);
		++wp:
	}
	else {
      fprintf(Logfile, "(Out of memory)\n");
      fprintf(stderr, "(Out of memory)\n");
	}
	return (void *)wp;
}


void delete(void *p, char *file, int line) {

	wrapper *wp = (wrapper *)p;
	fprintf(Logfile, "%s: %d ", file, line);
	if (!p) {
		fprintf(Logfile, 
		  "delete: NULL pointer argument\n");
		fprintf(stderr, 
		  "delete: NULL pointer argument\n");
		exit(1);
	}
	else {
		--wp;
		if (wp -> signature != MAGIC) {
           fprintf(Logfile,
              "delete: bad pntr arg (%p)\n", 
              wp);
           fprintf(stderr,
			  "delete: bad pntr arg (%p)\n", 
              wp);
			exit(1);
		}
		else {
			fprintf(Logfile,
		    "(Freeing mem @%p, alloc @%s: %d)\n",
			  wp, wp -> file_name, 
              wp -> line_number);
			wp -> signature = ~MAGIC;
			free(wp);
		}
	}
}

And here's the test program

#ifdef NEWMAIN
#include "new.h"

void main() {

	char *p = char *malloc();
	void *q = malloc;
	free(q);
	free(q);
	free(++p);
}
#endif

Interesting additions to new.h

#define malloc(size) new ((size), _FILE_, _LINE_)
#define free(p) delete((p), _FILE_, _LINE_)

void *new (size_t size, char *file, int line);
void delete(void *p, char *file, int line);

#define is_dynamic(p) ((wrapper *)(p))[-1].signature == MAGIC)

demonstration program

#include <new.h>

typedef struct link {
	int key;
	struct link *next;
} link;

void fred() {

	link *p;
	link *head;
	link *dynamic;
	link declared;

	dynamic = malloc(sizeof(link));
	head = dynamic;
	dynamic -> key = 0;
	dynamic -> next = &declared;
	declared -> key = 1;
	declared -> next = NULL;

	for (p = head; p; p = p -> next)
		if (is_dynamic(p)) delete(p);
}



