#ifndef	__LIST_H
#define __LIST_H

#include <stdlib.h>

typedef struct	_list_ele {
	struct	_list_ele	*next;
	struct	_list_ele	*prev;
	size_t	app_size;

	void (*copy)(void *this, void *src);
	void (*assign) (void *this, void *src);
	void (*print) (void *this);
	int	 (*equal) (void *this, void *key);
	void (*destruct) (void *this);
} _list_ele;

typedef struct	list {
	_list_ele	*head;
	_list_ele	*tail;
	int			num_ele;

	struct _list_ele	*current;
	int			index;
} list;

void _delete_chain(_list_ele *head);
int	 enqueue(list *this, void *usr_space);
void list_assign(list *this, const list *src);
void list_dup(list *this);
void *list_find(list *this, void *obj);
void *list_index(list *this, int n);
void list_print(list *this, int forward);
void *list_unlink(list *this, void *usr_space);
list *new_list(void);

void *new_list_ele(size_t app_size,
			void(*copy) (void *this, void *src),
			void(*assign) (void *this, void *src),
			void(*print) (void *this),
			int (*equal) (void *this, void *key),
			void(*destruct) (void *this),
			int	(*construct) (void *this, ...),
			...);

extern int	List_ele_err;
extern int	List_err;

#define	LIST_NOERR	0
#define LIST_NOMEM	1
#define LIST_NOINIT	2

#define FORWARD		1
#define BACKWARD	2

#define list_size(this) ((this)->num_ele)
#define list_empty(this) ((this)->head == NULL)
#define le_copy(this, src)									\
		(*((_list_ele*) (this)-1) -> copy) (this, src)
#define le_assign(this, src)								\
		(*((_list_ele*) (this)-1) -> assign) (this, src)
#define le_print(this)								   		\
		(*((_list_ele*) (this)-1) -> print) (this)
#define le_equal(this, key)									\
		(*((_list_ele*) (this)-1) -> equal) (this, key)
#define head(this)										 	\
		((this)->head ? (void*) ((this)->head +1) : NULL)
#define tail(this)										 	\
		((this)->tail ? (void*) ((this)->tail +1) : NULL)
#define push(this, p) enqueue((this), (p))
#define pop(this) list_unlink((this), tail(this))
#define dequeue(this) list_unlink((this), head(this))
#define delete_list(this)									\
		(_delete_chain((_list_ele*)this->head),				\
		free(this))
#define delete_list_ele(this)								\
		( (*((_list_ele*) (this))->destruct) ((app_area*)(this+1)),	\
		free((_list_ele*) (this) - sizeof(app_area)) )
#endif
